@@ -26,10 +26,12 @@ from typing import (
 from coverage import env
 from coverage.annotate import AnnotateReporter
 from coverage.collector import Collector, HAS_CTRACER
-from coverage.config import read_coverage_config
+from coverage.config import CoverageConfig, read_coverage_config
 from coverage.context import should_start_context_test_function, combine_context_switchers
 from coverage.data import CoverageData, combine_parallel_data
-from coverage.debug import DebugControl, short_stack, write_formatted_info
+from coverage.debug import (
+    DebugControl, NoDebugging, short_stack, write_formatted_info, relevant_environment_display
+)
 from coverage.disposition import disposition_debug_msg
 from coverage.exceptions import ConfigError, CoverageException, CoverageWarning, PluginError
 from coverage.files import PathAliases, abs_file, relative_filename, set_relative_directory
@@ -37,22 +39,21 @@ from coverage.html import HtmlReporter
 from coverage.inorout import InOrOut
 from coverage.jsonreport import JsonReporter
 from coverage.lcovreport import LcovReporter
-from coverage.misc import bool_or_none, join_regex, human_sorted
+from coverage.misc import bool_or_none, join_regex
 from coverage.misc import DefaultValue, ensure_dir_for_file, isolate_module
 from coverage.multiproc import patch_multiprocessing
 from coverage.plugin import FileReporter
 from coverage.plugin_support import Plugins
 from coverage.python import PythonFileReporter
-from coverage.report import render_report
+from coverage.report import SummaryReporter
+from coverage.report_core import render_report
 from coverage.results import Analysis
-from coverage.summary import SummaryReporter
 from coverage.types import (
-    TConfigurable, TConfigSectionIn, TConfigValueIn, TConfigValueOut,
+    FilePath, TConfigurable, TConfigSectionIn, TConfigValueIn, TConfigValueOut,
     TFileDisposition, TLineNo, TMorf,
 )
 from coverage.xmlreport import XmlReporter
 
-
 os = isolate_module(os)
 
 @contextlib.contextmanager
@@ -85,7 +86,14 @@ class Coverage(TConfigurable):
         cov.start()
         #.. call your code ..
         cov.stop()
-        cov.html_report(directory='covhtml')
+        cov.html_report(directory="covhtml")
+
+    A context manager is available to do the same thing::
+
+        cov = Coverage()
+        with cov.collect():
+            #.. call your code ..
+        cov.html_report(directory="covhtml")
 
     Note: in keeping with Python custom, names starting with underscore are
     not part of the public API. They might stop working at any point.  Please
@@ -114,13 +122,13 @@ class Coverage(TConfigurable):
 
     def __init__(                       # pylint: disable=too-many-arguments
         self,
-        data_file: Optional[Union[str, DefaultValue]] = DEFAULT_DATAFILE,
+        data_file: Optional[Union[FilePath, DefaultValue]] = DEFAULT_DATAFILE,
         data_suffix: Optional[Union[str, bool]] = None,
         cover_pylib: Optional[bool] = None,
         auto_data: bool = False,
         timid: Optional[bool] = None,
         branch: Optional[bool] = None,
-        config_file: Union[str, bool] = True,
+        config_file: Union[FilePath, bool] = True,
         source: Optional[Iterable[str]] = None,
         source_pkgs: Optional[Iterable[str]] = None,
         omit: Optional[Union[str, Iterable[str]]] = None,
@@ -219,11 +227,17 @@ class Coverage(TConfigurable):
             The `messages` parameter.
 
         """
+        # Start self.config as a usable default configuration. It will soon be
+        # replaced with the real configuration.
+        self.config = CoverageConfig()
+
         # data_file=None means no disk file at all. data_file missing means
         # use the value from the config file.
         self._no_disk = data_file is None
         if isinstance(data_file, DefaultValue):
             data_file = None
+        if data_file is not None:
+            data_file = os.fspath(data_file)
 
         # This is injectable by tests.
         self._debug_file: Optional[IO[str]] = None
@@ -241,14 +255,15 @@ class Coverage(TConfigurable):
         # A record of all the warnings that have been issued.
         self._warnings: List[str] = []
 
-        # Other instance attributes, set later.
-        self._debug: DebugControl
-        self._plugins: Plugins
-        self._inorout: InOrOut
-        self._data: CoverageData
-        self._collector: Collector
-        self._file_mapper: Callable[[str], str]
+        # Other instance attributes, set with placebos or placeholders.
+        # More useful objects will be created later.
+        self._debug: DebugControl = NoDebugging()
+        self._inorout: Optional[InOrOut] = None
+        self._plugins: Plugins = Plugins()
+        self._data: Optional[CoverageData] = None
+        self._collector: Optional[Collector] = None
 
+        self._file_mapper: Callable[[str], str] = abs_file
         self._data_suffix = self._run_suffix = None
         self._exclude_re: Dict[str, str] = {}
         self._old_sigterm: Optional[Callable[[int, Optional[FrameType]], Any]] = None
@@ -263,6 +278,8 @@ class Coverage(TConfigurable):
         self._should_write_debug = True
 
         # Build our configuration from a number of sources.
+        if not isinstance(config_file, bool):
+            config_file = os.fspath(config_file)
         self.config = read_coverage_config(
             config_file=config_file,
             warn=self._warn,
@@ -305,6 +322,8 @@ class Coverage(TConfigurable):
 
         # Create and configure the debugging controller.
         self._debug = DebugControl(self.config.debug, self._debug_file, self.config.debug_file)
+        if self._debug.should("process"):
+            self._debug.write("Coverage._init")
 
         if "multiprocessing" in (self.config.concurrency or ()):
             # Multi-processing uses parallel for the subprocesses, so also use
@@ -315,7 +334,8 @@ class Coverage(TConfigurable):
         self._exclude_re = {}
 
         set_relative_directory()
-        self._file_mapper = relative_filename if self.config.relative_files else abs_file
+        if self.config.relative_files:
+            self._file_mapper = relative_filename
 
         # Load plugins
         self._plugins = Plugins.load_plugins(self.config.plugins, self.config, self._debug)
@@ -334,10 +354,10 @@ class Coverage(TConfigurable):
             self._should_write_debug = False
             self._write_startup_debug()
 
-        # '[run] _crash' will raise an exception if the value is close by in
+        # "[run] _crash" will raise an exception if the value is close by in
         # the call stack, for testing error handling.
-        if self.config._crash and self.config._crash in short_stack(limit=4):
-            raise Exception(f"Crashing because called by {self.config._crash}")
+        if self.config._crash and self.config._crash in short_stack():
+            raise RuntimeError(f"Crashing because called by {self.config._crash}")
 
     def _write_startup_debug(self) -> None:
         """Write out debug info at startup if needed."""
@@ -369,8 +389,9 @@ class Coverage(TConfigurable):
         Calls `_should_trace_internal`, and returns the FileDisposition.
 
         """
+        assert self._inorout is not None
         disp = self._inorout.should_trace(filename, frame)
-        if self._debug.should('trace'):
+        if self._debug.should("trace"):
             self._debug.write(disposition_debug_msg(disp))
         return disp
 
@@ -380,8 +401,9 @@ class Coverage(TConfigurable):
         Returns a boolean: True if the file should be traced, False if not.
 
         """
+        assert self._inorout is not None
         reason = self._inorout.check_include_omit_etc(filename, frame)
-        if self._debug.should('trace'):
+        if self._debug.should("trace"):
             if not reason:
                 msg = f"Including {filename!r}"
             else:
@@ -400,9 +422,7 @@ class Coverage(TConfigurable):
 
         """
         if not self._no_warn_slugs:
-            # _warn() can be called before self.config is set in __init__...
-            if hasattr(self, "config"):
-                self._no_warn_slugs = list(self.config.disable_warnings)
+            self._no_warn_slugs = list(self.config.disable_warnings)
 
         if slug in self._no_warn_slugs:
             # Don't issue the warning
@@ -411,7 +431,7 @@ class Coverage(TConfigurable):
         self._warnings.append(msg)
         if slug:
             msg = f"{msg} ({slug})"
-        if hasattr(self, "_debug") and self._debug.should('pid'):
+        if self._debug.should("pid"):
             msg = f"[{os.getpid()}] {msg}"
         warnings.warn(msg, category=CoverageWarning, stacklevel=2)
 
@@ -477,13 +497,14 @@ class Coverage(TConfigurable):
     def load(self) -> None:
         """Load previously-collected coverage data from the data file."""
         self._init()
-        if hasattr(self, "_collector"):
+        if self._collector is not None:
             self._collector.reset()
         should_skip = self.config.parallel and not os.path.exists(self.config.data_file)
         if not should_skip:
             self._init_data(suffix=None)
         self._post_init()
         if not should_skip:
+            assert self._data is not None
             self._data.read()
 
     def _init_for_start(self) -> None:
@@ -535,6 +556,7 @@ class Coverage(TConfigurable):
 
         self._init_data(suffix)
 
+        assert self._data is not None
         self._collector.use_data(self._data, self.config.context)
 
         # Early warning if we aren't going to be able to support plugins.
@@ -555,7 +577,7 @@ class Coverage(TConfigurable):
         self._inorout = InOrOut(
             config=self.config,
             warn=self._warn,
-            debug=(self._debug if self._debug.should('trace') else None),
+            debug=(self._debug if self._debug.should("trace") else None),
             include_namespace_packages=self.config.include_namespace_packages,
         )
         self._inorout.plugins = self._plugins
@@ -578,7 +600,7 @@ class Coverage(TConfigurable):
 
     def _init_data(self, suffix: Optional[Union[str, bool]]) -> None:
         """Create a data file if we don't have one yet."""
-        if not hasattr(self, "_data"):
+        if self._data is None:
             # Create the data file.  We do this at construction time so that the
             # data file will be written into the directory where the process
             # started rather than wherever the process eventually chdir'd to.
@@ -594,13 +616,16 @@ class Coverage(TConfigurable):
     def start(self) -> None:
         """Start measuring code coverage.
 
-        Coverage measurement only occurs in functions called after
+        Coverage measurement is only collected in functions called after
         :meth:`start` is invoked.  Statements in the same scope as
         :meth:`start` won't be measured.
 
         Once you invoke :meth:`start`, you must also call :meth:`stop`
         eventually, or your process might not shut down cleanly.
 
+        The :meth:`collect` method is a context manager to handle both
+        starting and stopping collection.
+
         """
         self._init()
         if not self._inited_for_start:
@@ -608,6 +633,9 @@ class Coverage(TConfigurable):
             self._init_for_start()
         self._post_init()
 
+        assert self._collector is not None
+        assert self._inorout is not None
+
         # Issue warnings for possible problems.
         self._inorout.warn_conflicting_settings()
 
@@ -629,16 +657,30 @@ class Coverage(TConfigurable):
             if self._instances[-1] is self:
                 self._instances.pop()
         if self._started:
+            assert self._collector is not None
             self._collector.stop()
         self._started = False
 
+    @contextlib.contextmanager
+    def collect(self) -> Iterator[None]:
+        """A context manager to start/stop coverage measurement collection.
+
+        .. versionadded:: 7.3
+
+        """
+        self.start()
+        try:
+            yield
+        finally:
+            self.stop()
+
     def _atexit(self, event: str = "atexit") -> None:
         """Clean up on process shutdown."""
         if self._debug.should("process"):
             self._debug.write(f"{event}: pid: {os.getpid()}, instance: {self!r}")
         if self._started:
             self.stop()
-        if self._auto_save:
+        if self._auto_save or event == "sigterm":
             self.save()
 
     def _on_sigterm(self, signum_unused: int, frame_unused: Optional[FrameType]) -> None:
@@ -658,11 +700,12 @@ class Coverage(TConfigurable):
         """
         self._init()
         self._post_init()
-        if hasattr(self, "_collector"):
+        if self._collector is not None:
             self._collector.reset()
         self._init_data(suffix=None)
+        assert self._data is not None
         self._data.erase(parallel=self.config.parallel)
-        del self._data
+        self._data = None
         self._inited_for_start = False
 
     def switch_context(self, new_context: str) -> None:
@@ -681,18 +724,19 @@ class Coverage(TConfigurable):
         if not self._started:                           # pragma: part started
             raise CoverageException("Cannot switch context, coverage is not started")
 
+        assert self._collector is not None
         if self._collector.should_start_context:
             self._warn("Conflicting dynamic contexts", slug="dynamic-conflict", once=True)
 
         self._collector.switch_context(new_context)
 
-    def clear_exclude(self, which: str = 'exclude') -> None:
+    def clear_exclude(self, which: str = "exclude") -> None:
         """Clear the exclude list."""
         self._init()
         setattr(self.config, which + "_list", [])
         self._exclude_regex_stale()
 
-    def exclude(self, regex: str, which: str = 'exclude') -> None:
+    def exclude(self, regex: str, which: str = "exclude") -> None:
         """Exclude source lines from execution consideration.
 
         A number of lists of regular expressions are maintained.  Each list
@@ -709,7 +753,6 @@ class Coverage(TConfigurable):
         """
         self._init()
         excl_list = getattr(self.config, which + "_list")
-        assert isinstance(regex, str)
         excl_list.append(regex)
         self._exclude_regex_stale()
 
@@ -724,7 +767,7 @@ class Coverage(TConfigurable):
             self._exclude_re[which] = join_regex(excl_list)
         return self._exclude_re[which]
 
-    def get_exclude_list(self, which: str = 'exclude') -> List[str]:
+    def get_exclude_list(self, which: str = "exclude") -> List[str]:
         """Return a list of excluded regex strings.
 
         `which` indicates which list is desired.  See :meth:`exclude` for the
@@ -787,6 +830,7 @@ class Coverage(TConfigurable):
         self._post_init()
         self.get_data()
 
+        assert self._data is not None
         combine_parallel_data(
             self._data,
             aliases=self._make_aliases(),
@@ -810,13 +854,15 @@ class Coverage(TConfigurable):
         self._init_data(suffix=None)
         self._post_init()
 
-        for plugin in self._plugins:
-            if not plugin._coverage_enabled:
-                self._collector.plugin_was_disabled(plugin)
+        if self._collector is not None:
+            for plugin in self._plugins:
+                if not plugin._coverage_enabled:
+                    self._collector.plugin_was_disabled(plugin)
 
-        if hasattr(self, "_collector") and self._collector.flush_data():
-            self._post_save_work()
+            if self._collector.flush_data():
+                self._post_save_work()
 
+        assert self._data is not None
         return self._data
 
     def _post_save_work(self) -> None:
@@ -826,6 +872,9 @@ class Coverage(TConfigurable):
         Look for un-executed files.
 
         """
+        assert self._data is not None
+        assert self._inorout is not None
+
         # If there are still entries in the source_pkgs_unmatched list,
         # then we never encountered those packages.
         if self._warn_unimported_source:
@@ -837,13 +886,12 @@ class Coverage(TConfigurable):
 
         # Touch all the files that could have executed, so that we can
         # mark completely un-executed files as 0% covered.
-        if self._data is not None:
-            file_paths = collections.defaultdict(list)
-            for file_path, plugin_name in self._inorout.find_possibly_unexecuted_files():
-                file_path = self._file_mapper(file_path)
-                file_paths[plugin_name].append(file_path)
-            for plugin_name, paths in file_paths.items():
-                self._data.touch_files(paths, plugin_name)
+        file_paths = collections.defaultdict(list)
+        for file_path, plugin_name in self._inorout.find_possibly_unexecuted_files():
+            file_path = self._file_mapper(file_path)
+            file_paths[plugin_name].append(file_path)
+        for plugin_name, paths in file_paths.items():
+            self._data.touch_files(paths, plugin_name)
 
     # Backward compatibility with version 1.
     def analysis(self, morf: TMorf) -> Tuple[str, List[TLineNo], List[TLineNo], str]:
@@ -900,6 +948,7 @@ class Coverage(TConfigurable):
 
     def _get_file_reporter(self, morf: TMorf) -> FileReporter:
         """Get a FileReporter for a module or file name."""
+        assert self._data is not None
         plugin = None
         file_reporter: Union[str, FileReporter] = "python"
 
@@ -935,6 +984,7 @@ class Coverage(TConfigurable):
         measured is used to find the FileReporters.
 
         """
+        assert self._data is not None
         if not morfs:
             morfs = self._data.measured_files()
 
@@ -946,10 +996,11 @@ class Coverage(TConfigurable):
         return file_reporters
 
     def _prepare_data_for_reporting(self) -> None:
-        """Re-map data before reporting, to get implicit 'combine' behavior."""
+        """Re-map data before reporting, to get implicit "combine" behavior."""
         if self.config.paths:
             mapped_data = CoverageData(warn=self._warn, debug=self._debug, no_disk=True)
-            mapped_data.update(self._data, aliases=self._make_aliases())
+            if self._data is not None:
+                mapped_data.update(self._data, aliases=self._make_aliases())
             self._data = mapped_data
 
     def report(
@@ -1059,9 +1110,6 @@ class Coverage(TConfigurable):
         See :meth:`report` for other arguments.
 
         """
-        print("The annotate command will be removed in a future version.")
-        print("Get in touch if you still use it: ned@nedbatchelder.com")
-
         self._prepare_data_for_reporting()
         with override_config(
             self,
@@ -1214,10 +1262,10 @@ class Coverage(TConfigurable):
     ) -> float:
         """Generate an LCOV report of coverage results.
 
-        Each module in 'morfs' is included in the report. 'outfile' is the
+        Each module in `morfs` is included in the report. `outfile` is the
         path to write the file to, "-" will write to stdout.
 
-        See :meth 'report' for other arguments.
+        See :meth:`report` for other arguments.
 
         .. versionadded:: 6.3
         """
@@ -1251,41 +1299,34 @@ class Coverage(TConfigurable):
             return entries
 
         info = [
-            ('coverage_version', covmod.__version__),
-            ('coverage_module', covmod.__file__),
-            ('tracer', self._collector.tracer_name() if hasattr(self, "_collector") else "-none-"),
-            ('CTracer', 'available' if HAS_CTRACER else "unavailable"),
-            ('plugins.file_tracers', plugin_info(self._plugins.file_tracers)),
-            ('plugins.configurers', plugin_info(self._plugins.configurers)),
-            ('plugins.context_switchers', plugin_info(self._plugins.context_switchers)),
-            ('configs_attempted', self.config.attempted_config_files),
-            ('configs_read', self.config.config_files_read),
-            ('config_file', self.config.config_file),
-            ('config_contents',
-                repr(self.config._config_contents) if self.config._config_contents else '-none-'
+            ("coverage_version", covmod.__version__),
+            ("coverage_module", covmod.__file__),
+            ("tracer", self._collector.tracer_name() if self._collector is not None else "-none-"),
+            ("CTracer", "available" if HAS_CTRACER else "unavailable"),
+            ("plugins.file_tracers", plugin_info(self._plugins.file_tracers)),
+            ("plugins.configurers", plugin_info(self._plugins.configurers)),
+            ("plugins.context_switchers", plugin_info(self._plugins.context_switchers)),
+            ("configs_attempted", self.config.attempted_config_files),
+            ("configs_read", self.config.config_files_read),
+            ("config_file", self.config.config_file),
+            ("config_contents",
+                repr(self.config._config_contents) if self.config._config_contents else "-none-"
             ),
-            ('data_file', self._data.data_filename() if hasattr(self, "_data") else "-none-"),
-            ('python', sys.version.replace('\n', '')),
-            ('platform', platform.platform()),
-            ('implementation', platform.python_implementation()),
-            ('executable', sys.executable),
-            ('def_encoding', sys.getdefaultencoding()),
-            ('fs_encoding', sys.getfilesystemencoding()),
-            ('pid', os.getpid()),
-            ('cwd', os.getcwd()),
-            ('path', sys.path),
-            ('environment', human_sorted(
-                f"{k} = {v}"
-                for k, v in os.environ.items()
-                if (
-                    any(slug in k for slug in ("COV", "PY")) or
-                    (k in ("HOME", "TEMP", "TMP"))
-                )
-            )),
-            ('command_line', " ".join(getattr(sys, 'argv', ['-none-']))),
+            ("data_file", self._data.data_filename() if self._data is not None else "-none-"),
+            ("python", sys.version.replace("\n", "")),
+            ("platform", platform.platform()),
+            ("implementation", platform.python_implementation()),
+            ("executable", sys.executable),
+            ("def_encoding", sys.getdefaultencoding()),
+            ("fs_encoding", sys.getfilesystemencoding()),
+            ("pid", os.getpid()),
+            ("cwd", os.getcwd()),
+            ("path", sys.path),
+            ("environment", [f"{k} = {v}" for k, v in relevant_environment_display(os.environ)]),
+            ("command_line", " ".join(getattr(sys, "argv", ["-none-"]))),
         ]
 
-        if hasattr(self, "_inorout"):
+        if self._inorout is not None:
             info.extend(self._inorout.sys_info())
 
         info.extend(CoverageData.sys_info())
@@ -1295,12 +1336,12 @@ class Coverage(TConfigurable):
 
 # Mega debugging...
 # $set_env.py: COVERAGE_DEBUG_CALLS - Lots and lots of output about calls to Coverage.
-if int(os.environ.get("COVERAGE_DEBUG_CALLS", 0)):              # pragma: debugging
+if int(os.getenv("COVERAGE_DEBUG_CALLS", 0)):               # pragma: debugging
     from coverage.debug import decorate_methods, show_calls
 
     Coverage = decorate_methods(        # type: ignore[misc]
         show_calls(show_args=True),
-        butnot=['get_data']
+        butnot=["get_data"]
     )(Coverage)
 
 
@@ -1327,7 +1368,7 @@ def process_startup() -> Optional[Coverage]:
     not started by this call.
 
     """
-    cps = os.environ.get("COVERAGE_PROCESS_START")
+    cps = os.getenv("COVERAGE_PROCESS_START")
     if not cps:
         # No request for coverage, nothing to do.
         return None
