@@ -8,21 +8,23 @@ from __future__ import annotations
 import contextlib
 import functools
 import inspect
-import io
 import itertools
 import os
 import pprint
+import re
 import reprlib
 import sys
+import traceback
 import types
 import _thread
 
 from typing import (
-    Any, Callable, IO, Iterable, Iterator, Optional, List, Tuple, cast,
+    overload,
+    Any, Callable, IO, Iterable, Iterator, Mapping, Optional, List, Tuple,
 )
 
-from coverage.misc import isolate_module
-from coverage.types import TWritable
+from coverage.misc import human_sorted_items, isolate_module
+from coverage.types import AnyCallable, TWritable
 
 os = isolate_module(os)
 
@@ -37,7 +39,7 @@ FORCED_DEBUG_FILE = None
 class DebugControl:
     """Control and output for debugging."""
 
-    show_repr_attr = False      # For AutoReprMixin
+    show_repr_attr = False      # For auto_repr
 
     def __init__(
         self,
@@ -50,12 +52,17 @@ class DebugControl:
         self.suppress_callers = False
 
         filters = []
+        if self.should("process"):
+            filters.append(CwdTracker().filter)
+            filters.append(ProcessTracker().filter)
+        if self.should("pytest"):
+            filters.append(PytestTracker().filter)
         if self.should("pid"):
             filters.append(add_pid_and_tid)
+
         self.output = DebugOutputFile.get_one(
             output,
             file_name=file_name,
-            show_process=self.should("process"),
             filters=filters,
         )
         self.raw_output = self.output.outfile
@@ -79,13 +86,18 @@ class DebugControl:
         finally:
             self.suppress_callers = old
 
-    def write(self, msg: str) -> None:
+    def write(self, msg: str, *, exc: Optional[BaseException] = None) -> None:
         """Write a line of debug output.
 
         `msg` is the line to write. A newline will be appended.
 
+        If `exc` is provided, a stack trace of the exception will be written
+        after the message.
+
         """
-        self.output.write(msg+"\n")
+        self.output.write(msg + "\n")
+        if exc is not None:
+            self.output.write("".join(traceback.format_exception(None, exc, exc.__traceback__)))
         if self.should("self"):
             caller_self = inspect.stack()[1][0].f_locals.get("self")
             if caller_self is not None:
@@ -95,16 +107,6 @@ class DebugControl:
         self.output.flush()
 
 
-class DebugControlString(DebugControl):
-    """A `DebugControl` that writes to a StringIO, for testing."""
-    def __init__(self, options: Iterable[str]) -> None:
-        super().__init__(options, io.StringIO())
-
-    def get_output(self) -> str:
-        """Get the output text from the `DebugControl`."""
-        return cast(str, self.raw_output.getvalue())        # type: ignore
-
-
 class NoDebugging(DebugControl):
     """A replacement for DebugControl that will never try to do anything."""
     def __init__(self) -> None:
@@ -115,7 +117,7 @@ class NoDebugging(DebugControl):
         """Should we write debug messages?  Never."""
         return False
 
-    def write(self, msg: str) -> None:
+    def write(self, msg: str, *, exc: Optional[BaseException] = None) -> None:
         """This will never be called."""
         raise AssertionError("NoDebugging.write should never be called.")
 
@@ -170,37 +172,98 @@ def write_formatted_info(
         write(f" {line}")
 
 
-def short_stack(limit: Optional[int] = None, skip: int = 0) -> str:
+def exc_one_line(exc: Exception) -> str:
+    """Get a one-line summary of an exception, including class name and message."""
+    lines = traceback.format_exception_only(type(exc), exc)
+    return "|".join(l.rstrip() for l in lines)
+
+
+_FILENAME_REGEXES: List[Tuple[str, str]] = [
+    (r".*[/\\]pytest-of-.*[/\\]pytest-\d+([/\\]popen-gw\d+)?", "tmp:"),
+]
+_FILENAME_SUBS: List[Tuple[str, str]] = []
+
+@overload
+def short_filename(filename: str) -> str:
+    pass
+
+@overload
+def short_filename(filename: None) -> None:
+    pass
+
+def short_filename(filename: Optional[str]) -> Optional[str]:
+    """shorten a file name. Directories are replaced by prefixes like 'syspath:'"""
+    if not _FILENAME_SUBS:
+        for pathdir in sys.path:
+            _FILENAME_SUBS.append((pathdir, "syspath:"))
+        import coverage
+        _FILENAME_SUBS.append((os.path.dirname(coverage.__file__), "cov:"))
+        _FILENAME_SUBS.sort(key=(lambda pair: len(pair[0])), reverse=True)
+    if filename is not None:
+        for pat, sub in _FILENAME_REGEXES:
+            filename = re.sub(pat, sub, filename)
+        for before, after in _FILENAME_SUBS:
+            filename = filename.replace(before, after)
+    return filename
+
+
+def short_stack(
+    skip: int = 0,
+    full: bool = False,
+    frame_ids: bool = False,
+    short_filenames: bool = False,
+) -> str:
     """Return a string summarizing the call stack.
 
     The string is multi-line, with one line per stack frame. Each line shows
     the function name, the file name, and the line number:
 
         ...
-        start_import_stop : /Users/ned/coverage/trunk/tests/coveragetest.py @95
-        import_local_file : /Users/ned/coverage/trunk/tests/coveragetest.py @81
-        import_local_file : /Users/ned/coverage/trunk/coverage/backward.py @159
+        start_import_stop : /Users/ned/coverage/trunk/tests/coveragetest.py:95
+        import_local_file : /Users/ned/coverage/trunk/tests/coveragetest.py:81
+        import_local_file : /Users/ned/coverage/trunk/coverage/backward.py:159
         ...
 
-    `limit` is the number of frames to include, defaulting to all of them.
+    `skip` is the number of closest immediate frames to skip, so that debugging
+    functions can call this and not be included in the result.
 
-    `skip` is the number of frames to skip, so that debugging functions can
-    call this and not be included in the result.
+    If `full` is true, then include all frames.  Otherwise, initial "boring"
+    frames (ones in site-packages and earlier) are omitted.
+
+    `short_filenames` will shorten filenames using `short_filename`, to reduce
+    the amount of repetitive noise in stack traces.
 
     """
-    stack = inspect.stack()[limit:skip:-1]
-    return "\n".join("%30s : %s:%d" % (t[3], t[1], t[2]) for t in stack)
+    # Regexes in initial frames that we don't care about.
+    BORING_PRELUDE = [
+        "<string>",             # pytest-xdist has string execution.
+        r"\bigor.py$",          # Our test runner.
+        r"\bsite-packages\b",   # pytest etc getting to our tests.
+    ]
+
+    stack: Iterable[inspect.FrameInfo] = inspect.stack()[:skip:-1]
+    if not full:
+        for pat in BORING_PRELUDE:
+            stack = itertools.dropwhile(
+                (lambda fi, pat=pat: re.search(pat, fi.filename)),  # type: ignore[misc]
+                stack
+            )
+    lines = []
+    for frame_info in stack:
+        line = f"{frame_info.function:>30s} : "
+        if frame_ids:
+            line += f"{id(frame_info.frame):#x} "
+        filename = frame_info.filename
+        if short_filenames:
+            filename = short_filename(filename)
+        line += f"{filename}:{frame_info.lineno}"
+        lines.append(line)
+    return "\n".join(lines)
 
 
-def dump_stack_frames(
-    limit: Optional[int] = None,
-    out: Optional[TWritable] = None,
-    skip: int = 0
-) -> None:
-    """Print a summary of the stack to stdout, or someplace else."""
-    fout = out or sys.stdout
-    fout.write(short_stack(limit=limit, skip=skip+1))
-    fout.write("\n")
+def dump_stack_frames(out: TWritable, skip: int = 0) -> None:
+    """Print a summary of the stack to `out`."""
+    out.write(short_stack(skip=skip+1) + "\n")
 
 
 def clipped_repr(text: str, numchars: int = 50) -> str:
@@ -226,22 +289,21 @@ def add_pid_and_tid(text: str) -> str:
     return text
 
 
-class AutoReprMixin:
-    """A mixin implementing an automatic __repr__ for debugging."""
-    auto_repr_ignore = ["auto_repr_ignore", "$coverage.object_id"]
+AUTO_REPR_IGNORE = {"$coverage.object_id"}
 
-    def __repr__(self) -> str:
-        show_attrs = (
-            (k, v) for k, v in self.__dict__.items()
-            if getattr(v, "show_repr_attr", True)
-            and not callable(v)
-            and k not in self.auto_repr_ignore
-        )
-        return "<{klass} @0x{id:x} {attrs}>".format(
-            klass=self.__class__.__name__,
-            id=id(self),
-            attrs=" ".join(f"{k}={v!r}" for k, v in show_attrs),
-        )
+def auto_repr(self: Any) -> str:
+    """A function implementing an automatic __repr__ for debugging."""
+    show_attrs = (
+        (k, v) for k, v in self.__dict__.items()
+        if getattr(v, "show_repr_attr", True)
+        and not inspect.ismethod(v)
+        and k not in AUTO_REPR_IGNORE
+    )
+    return "<{klass} @{id:#x}{attrs}>".format(
+        klass=self.__class__.__name__,
+        id=id(self),
+        attrs="".join(f" {k}={v!r}" for k, v in show_attrs),
+    )
 
 
 def simplify(v: Any) -> Any:                                # pragma: debugging
@@ -266,7 +328,8 @@ def filter_text(text: str, filters: Iterable[Callable[[str], str]]) -> str:
     """Run `text` through a series of filters.
 
     `filters` is a list of functions. Each takes a string and returns a
-    string.  Each is run in turn.
+    string.  Each is run in turn. After each filter, the text is split into
+    lines, and each line is passed through the next filter.
 
     Returns: the final string that results after all of the filters have
     run.
@@ -275,10 +338,10 @@ def filter_text(text: str, filters: Iterable[Callable[[str], str]]) -> str:
     clean_text = text.rstrip()
     ending = text[len(clean_text):]
     text = clean_text
-    for fn in filters:
+    for filter_fn in filters:
         lines = []
         for line in text.splitlines():
-            lines.extend(fn(line).splitlines())
+            lines.extend(filter_fn(line).splitlines())
         text = "\n".join(lines)
     return text + ending
 
@@ -297,31 +360,65 @@ class CwdTracker:
         return text
 
 
+class ProcessTracker:
+    """Track process creation for debug logging."""
+    def __init__(self) -> None:
+        self.pid: int = os.getpid()
+        self.did_welcome = False
+
+    def filter(self, text: str) -> str:
+        """Add a message about how new processes came to be."""
+        welcome = ""
+        pid = os.getpid()
+        if self.pid != pid:
+            welcome = f"New process: forked {self.pid} -> {pid}\n"
+            self.pid = pid
+        elif not self.did_welcome:
+            argv = getattr(sys, "argv", None)
+            welcome = (
+                f"New process: {pid=}, executable: {sys.executable!r}\n"
+                + f"New process: cmd: {argv!r}\n"
+            )
+            if hasattr(os, "getppid"):
+                welcome += f"New process parent pid: {os.getppid()!r}\n"
+
+        if welcome:
+            self.did_welcome = True
+            return welcome + text
+        else:
+            return text
+
+
+class PytestTracker:
+    """Track the current pytest test name to add to debug messages."""
+    def __init__(self) -> None:
+        self.test_name: Optional[str] = None
+
+    def filter(self, text: str) -> str:
+        """Add a message when the pytest test changes."""
+        test_name = os.getenv("PYTEST_CURRENT_TEST")
+        if test_name != self.test_name:
+            text = f"Pytest context: {test_name}\n" + text
+            self.test_name = test_name
+        return text
+
+
 class DebugOutputFile:
     """A file-like object that includes pid and cwd information."""
     def __init__(
         self,
         outfile: Optional[IO[str]],
-        show_process: bool,
         filters: Iterable[Callable[[str], str]],
     ):
         self.outfile = outfile
-        self.show_process = show_process
         self.filters = list(filters)
-
-        if self.show_process:
-            self.filters.insert(0, CwdTracker().filter)
-            self.write(f"New process: executable: {sys.executable!r}\n")
-            self.write("New process: cmd: {!r}\n".format(getattr(sys, "argv", None)))
-            if hasattr(os, "getppid"):
-                self.write(f"New process: pid: {os.getpid()!r}, parent pid: {os.getppid()!r}\n")
+        self.pid = os.getpid()
 
     @classmethod
     def get_one(
         cls,
         fileobj: Optional[IO[str]] = None,
         file_name: Optional[str] = None,
-        show_process: bool = True,
         filters: Iterable[Callable[[str], str]] = (),
         interim: bool = False,
     ) -> DebugOutputFile:
@@ -333,9 +430,6 @@ class DebugOutputFile:
         provided, or COVERAGE_DEBUG_FILE, or stderr), and a process-wide
         singleton DebugOutputFile is made.
 
-        `show_process` controls whether the debug file adds process-level
-        information, and filters is a list of other message filters to apply.
-
         `filters` are the text filters to apply to the stream to annotate with
         pids, etc.
 
@@ -344,22 +438,25 @@ class DebugOutputFile:
         """
         if fileobj is not None:
             # Make DebugOutputFile around the fileobj passed.
-            return cls(fileobj, show_process, filters)
+            return cls(fileobj, filters)
 
         the_one, is_interim = cls._get_singleton_data()
         if the_one is None or is_interim:
             if file_name is not None:
                 fileobj = open(file_name, "a", encoding="utf-8")
             else:
-                file_name = os.environ.get("COVERAGE_DEBUG_FILE", FORCED_DEBUG_FILE)
+                file_name = os.getenv("COVERAGE_DEBUG_FILE", FORCED_DEBUG_FILE)
                 if file_name in ("stdout", "stderr"):
                     fileobj = getattr(sys, file_name)
                 elif file_name:
                     fileobj = open(file_name, "a", encoding="utf-8")
                 else:
                     fileobj = sys.stderr
-            the_one = cls(fileobj, show_process, filters)
+            the_one = cls(fileobj, filters)
             cls._set_singleton_data(the_one, interim)
+
+        if not(the_one.filters):
+            the_one.filters = list(filters)
         return the_one
 
     # Because of the way igor.py deletes and re-imports modules,
@@ -415,7 +512,7 @@ def decorate_methods(
     private: bool = False,
 ) -> Callable[..., Any]:                                    # pragma: debugging
     """A class decorator to apply a decorator to methods."""
-    def _decorator(cls):                                    # type: ignore
+    def _decorator(cls):                                    # type: ignore[no-untyped-def]
         for name, meth in inspect.getmembers(cls, inspect.isroutine):
             if name not in cls.__dict__:
                 continue
@@ -429,7 +526,7 @@ def decorate_methods(
     return _decorator
 
 
-def break_in_pudb(func: Callable[..., Any]) -> Callable[..., Any]:  # pragma: debugging
+def break_in_pudb(func: AnyCallable) -> AnyCallable:  # pragma: debugging
     """A function decorator to stop in the debugger for each call."""
     @functools.wraps(func)
     def _wrapper(*args: Any, **kwargs: Any) -> Any:
@@ -450,7 +547,7 @@ def show_calls(
     show_return: bool = False,
 ) -> Callable[..., Any]:                                    # pragma: debugging
     """A method decorator to debug-log each call to the function."""
-    def _decorator(func: Callable[..., Any]) -> Callable[..., Any]:
+    def _decorator(func: AnyCallable) -> AnyCallable:
         @functools.wraps(func)
         def _wrapper(self: Any, *args: Any, **kwargs: Any) -> Any:
             oid = getattr(self, OBJ_ID_ATTR, None)
@@ -469,7 +566,7 @@ def show_calls(
                 extra += ")"
             if show_stack:
                 extra += " @ "
-                extra += "; ".join(_clean_stack_line(l) for l in short_stack().splitlines())
+                extra += "; ".join(short_stack(short_filenames=True).splitlines())
             callid = next(CALLS)
             msg = f"{oid} {callid:04d} {func.__name__}{extra}\n"
             DebugOutputFile.get_one(interim=True).write(msg)
@@ -482,10 +579,32 @@ def show_calls(
     return _decorator
 
 
-def _clean_stack_line(s: str) -> str:                       # pragma: debugging
-    """Simplify some paths in a stack trace, for compactness."""
-    s = s.strip()
-    s = s.replace(os.path.dirname(__file__) + "/", "")
-    s = s.replace(os.path.dirname(os.__file__) + "/", "")
-    s = s.replace(sys.prefix + "/", "")
-    return s
+def relevant_environment_display(env: Mapping[str, str]) -> List[Tuple[str, str]]:
+    """Filter environment variables for a debug display.
+
+    Select variables to display (with COV or PY in the name, or HOME, TEMP, or
+    TMP), and also cloak sensitive values with asterisks.
+
+    Arguments:
+        env: a dict of environment variable names and values.
+
+    Returns:
+        A list of pairs (name, value) to show.
+
+    """
+    slugs = {"COV", "PY"}
+    include = {"HOME", "TEMP", "TMP"}
+    cloak = {"API", "TOKEN", "KEY", "SECRET", "PASS", "SIGNATURE"}
+
+    to_show = []
+    for name, val in env.items():
+        keep = False
+        if name in include:
+            keep = True
+        elif any(slug in name for slug in slugs):
+            keep = True
+        if keep:
+            if any(slug in name for slug in cloak):
+                val = re.sub(r"\w", "*", val)
+            to_show.append((name, val))
+    return human_sorted_items(to_show)
