@@ -29,6 +29,8 @@ PYPY = (platform.python_implementation() == "PyPy")
 
 # Python versions. We amend version_info with one more value, a zero if an
 # official version, or 1 if built from source beyond an official version.
+# Only use sys.version_info directly where tools like mypy need it to understand
+# version-specfic code, otherwise use PYVERSION.
 PYVERSION = sys.version_info + (int(platform.python_version()[-1] == "+"),)
 
 if PYPY:
@@ -119,12 +121,12 @@ class PYBEHAVIOR:
 C_TRACER = os.getenv("COVERAGE_TEST_TRACER", "c") == "c"
 
 # Are we coverage-measuring ourselves?
-METACOV = os.getenv("COVERAGE_COVERAGE", "") != ""
+METACOV = os.getenv("COVERAGE_COVERAGE") is not None
 
 # Are we running our test suite?
 # Even when running tests, you can use COVERAGE_TESTING=0 to disable the
 # test-specific behavior like AST checking.
-TESTING = os.getenv("COVERAGE_TESTING", "") == "True"
+TESTING = os.getenv("COVERAGE_TESTING") == "True"
 
 
 def debug_info() -> Iterable[Tuple[str, Any]]:
