@@ -15,7 +15,8 @@ import tokenize
 
 from types import CodeType
 from typing import (
-    cast, Any, Callable, Dict, Iterable, List, Optional, Sequence, Set, Tuple,
+    cast, Any, Callable, Dict, Iterable, List, Optional, Protocol, Sequence,
+    Set, Tuple,
 )
 
 from coverage import env
@@ -24,7 +25,7 @@ from coverage.debug import short_stack
 from coverage.exceptions import NoSource, NotPython
 from coverage.misc import join_regex, nice_pair
 from coverage.phystokens import generate_tokens
-from coverage.types import Protocol, TArc, TLineNo
+from coverage.types import TArc, TLineNo
 
 
 class PythonParser:
@@ -126,15 +127,25 @@ class PythonParser:
 
         # Tokenize, to find excluded suites, to find docstrings, and to find
         # multi-line statements.
-        indent = 0
-        exclude_indent = 0
-        excluding = False
-        excluding_decorators = False
-        prev_toktype = token.INDENT
-        first_line = None
-        empty = True
-        first_on_line = True
-        nesting = 0
+
+        # The last token seen. Start with INDENT to get module docstrings
+        prev_toktype: int = token.INDENT
+        # The current number of indents.
+        indent: int = 0
+        # An exclusion comment will exclude an entire clause at this indent.
+        exclude_indent: int = 0
+        # Are we currently excluding lines?
+        excluding: bool = False
+        # Are we excluding decorators now?
+        excluding_decorators: bool = False
+        # The line number of the first line in a multi-line statement.
+        first_line: int = 0
+        # Is the file empty?
+        empty: bool = True
+        # Is this the first token on a line?
+        first_on_line: bool = True
+        # Parenthesis (and bracket) nesting level.
+        nesting: int = 0
 
         assert self.text is not None
         tokgen = generate_tokens(self.text)
@@ -156,7 +167,10 @@ class PythonParser:
                     self.raw_classdefs.add(slineno)
             elif toktype == token.OP:
                 if ttext == ":" and nesting == 0:
-                    should_exclude = (elineno in self.raw_excluded) or excluding_decorators
+                    should_exclude = (
+                        self.raw_excluded.intersection(range(first_line, elineno + 1))
+                        or excluding_decorators
+                    )
                     if not excluding and should_exclude:
                         # Start excluding a suite.  We trigger off of the colon
                         # token so that the #pragma comment will be recognized on
@@ -175,26 +189,26 @@ class PythonParser:
                     nesting += 1
                 elif ttext in ")]}":
                     nesting -= 1
-            elif toktype == token.STRING and prev_toktype == token.INDENT:
-                # Strings that are first on an indented line are docstrings.
-                # (a trick from trace.py in the stdlib.) This works for
-                # 99.9999% of cases.  For the rest (!) see:
-                # http://stackoverflow.com/questions/1769332/x/1769794#1769794
-                self.raw_docstrings.update(range(slineno, elineno+1))
+            elif toktype == token.STRING:
+                if prev_toktype == token.INDENT:
+                    # Strings that are first on an indented line are docstrings.
+                    # (a trick from trace.py in the stdlib.) This works for
+                    # 99.9999% of cases.
+                    self.raw_docstrings.update(range(slineno, elineno+1))
             elif toktype == token.NEWLINE:
-                if first_line is not None and elineno != first_line:    # type: ignore[unreachable]
+                if first_line and elineno != first_line:
                     # We're at the end of a line, and we've ended on a
                     # different line than the first line of the statement,
                     # so record a multi-line range.
-                    for l in range(first_line, elineno+1):              # type: ignore[unreachable]
+                    for l in range(first_line, elineno+1):
                         self._multiline[l] = first_line
-                first_line = None
+                first_line = 0
                 first_on_line = True
 
             if ttext.strip() and toktype != tokenize.COMMENT:
                 # A non-white-space token.
                 empty = False
-                if first_line is None:
+                if not first_line:
                     # The token is not white space, and is the first in a statement.
                     first_line = slineno
                     # Check whether to end an excluded suite.
@@ -251,7 +265,7 @@ class PythonParser:
         """
         try:
             self._raw_parse()
-        except (tokenize.TokenError, IndentationError) as err:
+        except (tokenize.TokenError, IndentationError, SyntaxError) as err:
             if hasattr(err, "lineno"):
                 lineno = err.lineno         # IndentationError
             else:
@@ -425,7 +439,7 @@ class ByteParser:
                         yield line_num
                         last_line_num = line_num
                     byte_num += byte_incr
-                if env.PYBEHAVIOR.negative_lnotab and line_incr >= 0x80:
+                if line_incr >= 0x80:
                     line_incr -= 0x100
                 line_num += line_incr
             if line_num != last_line_num:
@@ -675,13 +689,16 @@ class AstArcAnalyzer:
 
         # Turn on AST dumps with an environment variable.
         # $set_env.py: COVERAGE_AST_DUMP - Dump the AST nodes when parsing code.
-        dump_ast = bool(int(os.environ.get("COVERAGE_AST_DUMP", 0)))
+        dump_ast = bool(int(os.getenv("COVERAGE_AST_DUMP", "0")))
 
         if dump_ast:                                # pragma: debugging
             # Dump the AST so that failing tests have helpful output.
             print(f"Statements: {self.statements}")
             print(f"Multiline map: {self.multiline}")
-            ast_dump(self.root_node)
+            dumpkw: Dict[str, Any] = {}
+            if sys.version_info >= (3, 9):
+                dumpkw["indent"] = 4
+            print(ast.dump(self.root_node, include_attributes=True, **dumpkw))
 
         self.arcs: Set[TArc] = set()
 
@@ -694,7 +711,7 @@ class AstArcAnalyzer:
         self.block_stack: List[Block] = []
 
         # $set_env.py: COVERAGE_TRACK_ARCS - Trace possible arcs added while parsing code.
-        self.debug = bool(int(os.environ.get("COVERAGE_TRACK_ARCS", 0)))
+        self.debug = bool(int(os.getenv("COVERAGE_TRACK_ARCS", "0")))
 
     def analyze(self) -> None:
         """Examine the AST tree from `root_node` to determine possible arcs.
@@ -719,7 +736,7 @@ class AstArcAnalyzer:
         """Add an arc, including message fragments to use if it is missing."""
         if self.debug:                      # pragma: debugging
             print(f"\nAdding possible arc: ({start}, {end}): {smsg!r}, {emsg!r}")
-            print(short_stack(limit=10))
+            print(short_stack())
         self.arcs.add((start, end))
 
         if smsg is not None or emsg is not None:
@@ -747,10 +764,10 @@ class AstArcAnalyzer:
 
     def _line_decorated(self, node: ast.FunctionDef) -> TLineNo:
         """Compute first line number for things that can be decorated (classes and functions)."""
-        lineno = node.lineno
-        if env.PYBEHAVIOR.trace_decorated_def or env.PYBEHAVIOR.def_ast_no_decorator:
-            if node.decorator_list:
-                lineno = node.decorator_list[0].lineno
+        if node.decorator_list:
+            lineno = node.decorator_list[0].lineno
+        else:
+            lineno = node.lineno
         return lineno
 
     def _line__Assign(self, node: ast.Assign) -> TLineNo:
@@ -1007,17 +1024,15 @@ class AstArcAnalyzer:
         last: Optional[TLineNo] = node.lineno
         decs = node.decorator_list
         if decs:
-            if env.PYBEHAVIOR.trace_decorated_def or env.PYBEHAVIOR.def_ast_no_decorator:
-                last = None
+            last = None
             for dec_node in decs:
                 dec_start = self.line_for_node(dec_node)
-                if last is not None and dec_start != last:
-                    self.add_arc(last, dec_start)
+                if last is not None and dec_start != last:  # type: ignore[unreachable]
+                    self.add_arc(last, dec_start)           # type: ignore[unreachable]
                 last = dec_start
             assert last is not None
-            if env.PYBEHAVIOR.trace_decorated_def:
-                self.add_arc(last, main_line)
-                last = main_line
+            self.add_arc(last, main_line)
+            last = main_line
             if env.PYBEHAVIOR.trace_decorator_line_again:
                 for top, bot in zip(decs, decs[1:]):
                     self.add_arc(self.line_for_node(bot), self.line_for_node(top))
@@ -1030,10 +1045,6 @@ class AstArcAnalyzer:
             if node.body:
                 body_start = self.line_for_node(node.body[0])
                 body_start = self.multiline.get(body_start, body_start)
-                for lineno in range(last+1, body_start):
-                    if lineno in self.statements:
-                        self.add_arc(last, lineno)
-                        last = lineno
         # The body is handled in collect_arcs.
         assert last is not None
         return {ArcStart(last)}
@@ -1343,77 +1354,7 @@ class AstArcAnalyzer:
 
     _code_object__Lambda = _make_expression_code_method("lambda")
     _code_object__GeneratorExp = _make_expression_code_method("generator expression")
-    _code_object__DictComp = _make_expression_code_method("dictionary comprehension")
-    _code_object__SetComp = _make_expression_code_method("set comprehension")
-    _code_object__ListComp = _make_expression_code_method("list comprehension")
-
-
-# Code only used when dumping the AST for debugging.
-
-SKIP_DUMP_FIELDS = ["ctx"]
-
-def _is_simple_value(value: Any) -> bool:
-    """Is `value` simple enough to be displayed on a single line?"""
-    return (
-        value in [None, [], (), {}, set(), frozenset(), Ellipsis] or
-        isinstance(value, (bytes, int, float, str))
-    )
-
-def ast_dump(
-    node: ast.AST,
-    depth: int = 0,
-    print: Callable[[str], None] = print,   # pylint: disable=redefined-builtin
-) -> None:
-    """Dump the AST for `node`.
-
-    This recursively walks the AST, printing a readable version.
-
-    """
-    indent = " " * depth
-    lineno = getattr(node, "lineno", None)
-    if lineno is not None:
-        linemark = f" @ {node.lineno},{node.col_offset}"
-        if hasattr(node, "end_lineno"):
-            assert hasattr(node, "end_col_offset")
-            linemark += ":"
-            if node.end_lineno != node.lineno:
-                linemark += f"{node.end_lineno},"
-            linemark += f"{node.end_col_offset}"
-    else:
-        linemark = ""
-    head = f"{indent}<{node.__class__.__name__}{linemark}"
-
-    named_fields = [
-        (name, value)
-        for name, value in ast.iter_fields(node)
-        if name not in SKIP_DUMP_FIELDS
-    ]
-    if not named_fields:
-        print(f"{head}>")
-    elif len(named_fields) == 1 and _is_simple_value(named_fields[0][1]):
-        field_name, value = named_fields[0]
-        print(f"{head} {field_name}: {value!r}>")
-    else:
-        print(head)
-        if 0:
-            print("{}# mro: {}".format(     # type: ignore[unreachable]
-                indent, ", ".join(c.__name__ for c in node.__class__.__mro__[1:]),
-            ))
-        next_indent = indent + "    "
-        for field_name, value in named_fields:
-            prefix = f"{next_indent}{field_name}:"
-            if _is_simple_value(value):
-                print(f"{prefix} {value!r}")
-            elif isinstance(value, list):
-                print(f"{prefix} [")
-                for n in value:
-                    if _is_simple_value(n):
-                        print(f"{next_indent}    {n!r}")
-                    else:
-                        ast_dump(n, depth + 8, print=print)
-                print(f"{next_indent}]")
-            else:
-                print(prefix)
-                ast_dump(value, depth + 8, print=print)
-
-        print(f"{indent}>")
+    if env.PYBEHAVIOR.comprehensions_are_functions:
+        _code_object__DictComp = _make_expression_code_method("dictionary comprehension")
+        _code_object__SetComp = _make_expression_code_method("set comprehension")
+        _code_object__ListComp = _make_expression_code_method("list comprehension")
