@@ -52,7 +52,6 @@ class PyTracer(TTracer):
     # used to force the use of this tracer.
 
     def __init__(self) -> None:
-        # pylint: disable=super-init-not-called
         # Attributes set from the collector:
         self.data: TTraceData
         self.trace_arcs = False
@@ -85,10 +84,9 @@ class PyTracer(TTracer):
         self._cached_bound_method_trace: TTraceFn = self._trace
 
     def __repr__(self) -> str:
-        me = id(self)
         points = sum(len(v) for v in self.data.values())
         files = len(self.data)
-        return f"<PyTracer at 0x{me:x}: {points} data points in {files} files>"
+        return f"<PyTracer at {id(self):#x}: {points} data points in {files} files>"
 
     def log(self, marker: str, *args: Any) -> None:
         """For hard-core logging of what this tracer is doing."""
@@ -198,7 +196,7 @@ class PyTracer(TTracer):
                     tracename = disp.source_filename
                     assert tracename is not None
                     if tracename not in self.data:
-                        self.data[tracename] = set()    # type: ignore[assignment]
+                        self.data[tracename] = set()
                     self.cur_file_data = self.data[tracename]
                 else:
                     frame.f_trace_lines = False
@@ -316,7 +314,7 @@ class PyTracer(TTracer):
             # PyPy clears the trace function before running atexit functions,
             # so don't warn if we are in atexit on PyPy and the trace function
             # has changed to None.
-            dont_warn = (env.PYPY and env.PYPYVERSION >= (5, 4) and self.in_atexit and tf is None)
+            dont_warn = (env.PYPY and self.in_atexit and tf is None)
             if (not dont_warn) and tf != self._cached_bound_method_trace:   # pylint: disable=comparison-with-callable
                 self.warn(
                     "Trace function changed, data is likely wrong: " +
