@@ -23,7 +23,7 @@ from coverage import env
 from coverage.files import set_relative_directory
 
 # Pytest will rewrite assertions in test modules, but not elsewhere.
-# This tells pytest to also rewrite assertions in coveragetest.py.
+# This tells pytest to also rewrite assertions in these files:
 pytest.register_assert_rewrite("tests.coveragetest")
 pytest.register_assert_rewrite("tests.helpers")
 
@@ -40,7 +40,7 @@ def set_warnings() -> None:
     warnings.simplefilter("once", DeprecationWarning)
 
     # Warnings to suppress:
-    # How come these warnings are successfully suppressed here, but not in setup.cfg??
+    # How come these warnings are successfully suppressed here, but not in pyproject.toml??
 
     warnings.filterwarnings(
         "ignore",
@@ -87,7 +87,7 @@ def reset_filesdotpy_globals() -> Iterator[None]:
     set_relative_directory()
     yield
 
-WORKER = os.environ.get("PYTEST_XDIST_WORKER", "none")
+WORKER = os.getenv("PYTEST_XDIST_WORKER", "none")
 
 def pytest_sessionstart() -> None:
     """Run once at the start of the test session."""
