@@ -87,14 +87,9 @@ class CoverageTest(
         The imported module is returned.
 
         """
-        cov.start()
-        try:                                    # pragma: nested
+        with cov.collect():
             # Import the Python file, executing it.
-            mod = import_local_file(modname, modfile)
-        finally:                                # pragma: nested
-            # Stop coverage.py.
-            cov.stop()
-        return mod
+            return import_local_file(modname, modfile)
 
     def get_report(self, cov: Coverage, squeeze: bool = True, **kwargs: Any) -> str:
         """Get the report from `cov`, and canonicalize it."""
@@ -198,11 +193,11 @@ class CoverageTest(
         # Get the analysis results, and check that they are right.
         analysis = cov._analyze(mod)
         statements = sorted(analysis.statements)
-        if lines is not None:
+        if lines:
             if isinstance(lines[0], int):
                 # lines is just a list of numbers, it must match the statements
                 # found in the code.
-                assert statements == lines, f"{statements!r} != {lines!r}"
+                assert statements == lines, f"lines: {statements!r} != {lines!r}"
             else:
                 # lines is a list of possible line number lists, one of them
                 # must match.
@@ -214,7 +209,7 @@ class CoverageTest(
 
             missing_formatted = analysis.missing_formatted()
             if isinstance(missing, str):
-                msg = f"{missing_formatted!r} != {missing!r}"
+                msg = f"missing: {missing_formatted!r} != {missing!r}"
                 assert missing_formatted == missing, msg
             else:
                 for missing_list in missing:
@@ -293,7 +288,7 @@ class CoverageTest(
         def capture_warning(
             msg: str,
             slug: Optional[str] = None,
-            once: bool = False,               # pylint: disable=unused-argument
+            once: bool = False,                 # pylint: disable=unused-argument
         ) -> None:
             """A fake implementation of Coverage._warn, to capture warnings."""
             # NOTE: we don't implement `once`.
@@ -302,7 +297,7 @@ class CoverageTest(
             saved_warnings.append(msg)
 
         original_warn = cov._warn
-        cov._warn = capture_warning                 # type: ignore[assignment]
+        cov._warn = capture_warning             # type: ignore[method-assign]
 
         try:
             yield
@@ -327,7 +322,7 @@ class CoverageTest(
                 if saved_warnings:
                     assert False, f"Unexpected warnings: {saved_warnings!r}"
         finally:
-            cov._warn = original_warn               # type: ignore[assignment]
+            cov._warn = original_warn           # type: ignore[method-assign]
 
     def assert_same_files(self, flist1: Iterable[str], flist2: Iterable[str]) -> None:
         """Assert that `flist1` and `flist2` are the same set of file names."""
@@ -443,22 +438,18 @@ class CoverageTest(
 
         cmd = " ".join([shlex.quote(w) for w in command_words] + command_args)
 
-        # Add our test modules directory to PYTHONPATH.  I'm sure there's too
-        # much path munging here, but...
-        pythonpath_name = "PYTHONPATH"
-
-        testmods = nice_file(self.working_root(), "tests/modules")
-        zipfile = nice_file(self.working_root(), "tests/zipmods.zip")
-        pypath = os.getenv(pythonpath_name, '')
-        if pypath:
-            pypath += os.pathsep
-        pypath += testmods + os.pathsep + zipfile
-        self.set_environ(pythonpath_name, pypath)
-
         self.last_command_status, self.last_command_output = run_command(cmd)
         print(self.last_command_output)
         return self.last_command_status, self.last_command_output
 
+    def add_test_modules_to_pythonpath(self) -> None:
+        """Add our test modules directory to PYTHONPATH."""
+        # Check that there isn't already a PYTHONPATH.
+        assert os.getenv("PYTHONPATH") is None
+        testmods = nice_file(self.working_root(), "tests/modules")
+        zipfile = nice_file(self.working_root(), "tests/zipmods.zip")
+        self.set_environ("PYTHONPATH", testmods + os.pathsep + zipfile)
+
     def working_root(self) -> str:
         """Where is the root of the coverage.py working tree?"""
         return os.path.dirname(nice_file(__file__, ".."))
