@@ -7,6 +7,8 @@ from __future__ import annotations
 
 import collections
 import contextlib
+import dis
+import io
 import os
 import os.path
 import re
@@ -17,13 +19,14 @@ import warnings
 
 from pathlib import Path
 from typing import (
-    Any, Callable, Iterable, Iterator, List, Optional, Set, Tuple, Type,
+    Any, Callable, Iterable, Iterator, List, NoReturn, Optional, Set, Tuple, Type,
     TypeVar, Union, cast,
 )
 
 import pytest
 
 from coverage import env
+from coverage.debug import DebugControl
 from coverage.exceptions import CoverageWarning
 from coverage.misc import output_encoding
 from coverage.types import TArc, TLineNo
@@ -39,7 +42,7 @@ def run_command(cmd: str) -> Tuple[int, str]:
     # the test suite.  Use these lines to get a list of the tests using them:
     if 0:  # pragma: debugging
         with open("/tmp/processes.txt", "a") as proctxt:  # type: ignore[unreachable]
-            print(os.environ.get("PYTEST_CURRENT_TEST", "unknown"), file=proctxt, flush=True)
+            print(os.getenv("PYTEST_CURRENT_TEST", "unknown"), file=proctxt, flush=True)
 
     # In some strange cases (PyPy3 in a virtualenv!?) the stdout encoding of
     # the subprocess is set incorrectly to ascii.  Use an environment variable
@@ -63,6 +66,9 @@ def run_command(cmd: str) -> Tuple[int, str]:
     return status, output_str
 
 
+# $set_env.py: COVERAGE_DIS - Disassemble test code to /tmp/dis
+SHOW_DIS = bool(int(os.getenv("COVERAGE_DIS", "0")))
+
 def make_file(
     filename: str,
     text: str = "",
@@ -94,14 +100,27 @@ def make_file(
         data = text.encode("utf-8")
 
     # Make sure the directories are available.
-    dirs, _ = os.path.split(filename)
-    if dirs and not os.path.exists(dirs):
-        os.makedirs(dirs)
+    dirs, basename = os.path.split(filename)
+    if dirs:
+        os.makedirs(dirs, exist_ok=True)
 
     # Create the file.
     with open(filename, 'wb') as f:
         f.write(data)
 
+    if text and basename.endswith(".py") and SHOW_DIS:      # pragma: debugging
+        os.makedirs("/tmp/dis", exist_ok=True)
+        with open(f"/tmp/dis/{basename}.dis", "w") as fdis:
+            print(f"# {os.path.abspath(filename)}", file=fdis)
+            cur_test = os.getenv("PYTEST_CURRENT_TEST", "unknown")
+            print(f"# PYTEST_CURRENT_TEST = {cur_test}", file=fdis)
+            try:
+                dis.dis(text, file=fdis)
+            except Exception as exc:
+                # Some tests make .py files that aren't Python, so dis will
+                # fail, which is expected.
+                print(f"#! {exc!r}", file=fdis)
+
     # For debugging, enable this to show the contents of files created.
     if 0:  # pragma: debugging
         print(f"   ───┬──┤ {filename} ├───────────────────────")  # type: ignore[unreachable]
@@ -337,3 +356,43 @@ xfail_pypy38 = pytest.mark.xfail(
     env.PYPY and env.PYVERSION[:2] == (3, 8) and env.PYPYVERSION < (7, 3, 11),
     reason="These tests fail on older PyPy 3.8",
 )
+
+
+class FailingProxy:
+    """A proxy for another object, but one method will fail a few times before working."""
+    def __init__(self, obj: Any, methname: str, fails: List[Exception]) -> None:
+        """Create the failing proxy.
+
+        `obj` is the object to proxy.  `methname` is the method that will fail
+        a few times.  `fails` are the exceptions to fail with. Once used up,
+        the method will proxy correctly.
+
+        """
+        self.obj = obj
+        self.methname = methname
+        self.fails = fails
+
+    def __getattr__(self, name: str) -> Any:
+        if name == self.methname and self.fails:
+            meth = self._make_failing_method(self.fails[0])
+            del self.fails[0]
+        else:
+            meth = getattr(self.obj, name)
+        return meth
+
+    def _make_failing_method(self, exc: Exception) -> Callable[..., NoReturn]:
+        """Return a function that will raise `exc`."""
+        def _meth(*args: Any, **kwargs: Any) -> NoReturn:
+            raise exc
+        return _meth
+
+
+class DebugControlString(DebugControl):
+    """A `DebugControl` that writes to a StringIO, for testing."""
+    def __init__(self, options: Iterable[str]) -> None:
+        self.io = io.StringIO()
+        super().__init__(options, self.io)
+
+    def get_output(self) -> str:
+        """Get the output text from the `DebugControl`."""
+        return self.io.getvalue()
