@@ -25,11 +25,10 @@ from coverage.data import line_counts, sorted_lines
 from coverage.exceptions import CoverageException, DataError, NoDataError, NoSource
 from coverage.files import abs_file, relative_filename
 from coverage.misc import import_local_file
-from coverage.types import FilePathClasses, FilePathType, Protocol, TCovKwargs
+from coverage.types import FilePathClasses, FilePathType, TCovKwargs
 
 from tests.coveragetest import CoverageTest, TESTS_DIR, UsingModulesMixin
-from tests.goldtest import contains, doesnt_contain
-from tests.helpers import arcz_to_arcs, assert_count_equal, assert_coverage_warnings
+from tests.helpers import assert_count_equal, assert_coverage_warnings
 from tests.helpers import change_dir, nice_file, os_sep
 
 BAD_SQLITE_REGEX = r"file( is encrypted or)? is not a database"
@@ -283,9 +282,8 @@ class ApiTest(CoverageTest):
 
         def run_one_function(f: Callable[[], None]) -> None:
             cov.erase()
-            cov.start()
-            f()
-            cov.stop()
+            with cov.collect():
+                f()
 
             fs = cov.get_data().measured_files()
             lines.append(cov.get_data().lines(list(fs)[0]))
@@ -364,30 +362,26 @@ class ApiTest(CoverageTest):
     def test_start_save_stop(self) -> None:
         self.make_code1_code2()
         cov = coverage.Coverage()
-        cov.start()
-        import_local_file("code1")                                     # pragma: nested
-        cov.save()                                                     # pragma: nested
-        import_local_file("code2")                                     # pragma: nested
-        cov.stop()                                                     # pragma: nested
+        with cov.collect():
+            import_local_file("code1")
+            cov.save()
+            import_local_file("code2")
         self.check_code1_code2(cov)
 
     def test_start_save_nostop(self) -> None:
         self.make_code1_code2()
         cov = coverage.Coverage()
-        cov.start()
-        import_local_file("code1")                                     # pragma: nested
-        cov.save()                                                     # pragma: nested
-        import_local_file("code2")                                     # pragma: nested
-        self.check_code1_code2(cov)                                    # pragma: nested
-        # Then stop it, or the test suite gets out of whack.
-        cov.stop()                                                     # pragma: nested
+        with cov.collect():
+            import_local_file("code1")
+            cov.save()
+            import_local_file("code2")
+            self.check_code1_code2(cov)
 
     def test_two_getdata_only_warn_once(self) -> None:
         self.make_code1_code2()
         cov = coverage.Coverage(source=["."], omit=["code1.py"])
-        cov.start()
-        import_local_file("code1")                                     # pragma: nested
-        cov.stop()                                                     # pragma: nested
+        with cov.collect():
+            import_local_file("code1")
         # We didn't collect any data, so we should get a warning.
         with self.assert_warnings(cov, ["No data was collected"]):
             cov.get_data()
@@ -399,17 +393,15 @@ class ApiTest(CoverageTest):
     def test_two_getdata_warn_twice(self) -> None:
         self.make_code1_code2()
         cov = coverage.Coverage(source=["."], omit=["code1.py", "code2.py"])
-        cov.start()
-        import_local_file("code1")                                     # pragma: nested
-        # We didn't collect any data, so we should get a warning.
-        with self.assert_warnings(cov, ["No data was collected"]):     # pragma: nested
-            cov.save()                                                 # pragma: nested
-        import_local_file("code2")                                     # pragma: nested
-        # Calling get_data a second time after tracing some more will warn again.
-        with self.assert_warnings(cov, ["No data was collected"]):     # pragma: nested
-            cov.get_data()                                             # pragma: nested
-        # Then stop it, or the test suite gets out of whack.
-        cov.stop()                                                     # pragma: nested
+        with cov.collect():
+            import_local_file("code1")
+            # We didn't collect any data, so we should get a warning.
+            with self.assert_warnings(cov, ["No data was collected"]):
+                cov.save()
+            import_local_file("code2")
+            # Calling get_data a second time after tracing some more will warn again.
+            with self.assert_warnings(cov, ["No data was collected"]):
+                cov.get_data()
 
     def make_good_data_files(self) -> None:
         """Make some good data files."""
@@ -610,11 +602,34 @@ class ApiTest(CoverageTest):
             """)
         assert expected == self.stdout()
 
-    def make_test_files(self) -> None:
-        """Create a simple file representing a method with two tests.
+    def test_config_crash(self) -> None:
+        # The internal '[run] _crash' setting can be used to artificially raise
+        # exceptions from inside Coverage.
+        cov = coverage.Coverage()
+        cov.set_option("run:_crash", "test_config_crash")
+        with pytest.raises(Exception, match="Crashing because called by test_config_crash"):
+            cov.start()
 
-        Returns absolute path to the file.
-        """
+    def test_config_crash_no_crash(self) -> None:
+        # '[run] _crash' really checks the call stack.
+        cov = coverage.Coverage()
+        cov.set_option("run:_crash", "not_my_caller")
+        cov.start()
+        cov.stop()
+
+    def test_run_debug_sys(self) -> None:
+        # https://github.com/nedbat/coveragepy/issues/907
+        cov = coverage.Coverage()
+        with cov.collect():
+            d = dict(cov.sys_info())
+        assert cast(str, d['data_file']).endswith(".coverage")
+
+
+class SwitchContextTest(CoverageTest):
+    """Tests of the .switch_context() method."""
+
+    def make_test_files(self) -> None:
+        """Create a simple file representing a method with two tests."""
         self.make_file("testsuite.py", """\
             def timestwo(x):
                 return x*2
@@ -633,9 +648,7 @@ class ApiTest(CoverageTest):
 
         # Test runner starts
         cov = coverage.Coverage()
-        cov.start()
-
-        if "pragma: nested":
+        with cov.collect():
             # Imports the test suite
             suite = import_local_file("testsuite")
 
@@ -649,7 +662,6 @@ class ApiTest(CoverageTest):
 
             # Runner finishes
             cov.save()
-            cov.stop()
 
         # Labeled data is collected
         data = cov.get_data()
@@ -671,9 +683,7 @@ class ApiTest(CoverageTest):
 
         # Test runner starts
         cov = coverage.Coverage(context="mysuite")
-        cov.start()
-
-        if "pragma: nested":
+        with cov.collect():
             # Imports the test suite
             suite = import_local_file("testsuite")
 
@@ -687,7 +697,6 @@ class ApiTest(CoverageTest):
 
             # Runner finishes
             cov.save()
-            cov.stop()
 
         # Labeled data is collected
         data = cov.get_data()
@@ -705,12 +714,11 @@ class ApiTest(CoverageTest):
     def test_dynamic_context_conflict(self) -> None:
         cov = coverage.Coverage(source=["."])
         cov.set_option("run:dynamic_context", "test_function")
-        cov.start()
-        with pytest.warns(Warning) as warns:
-            # Switch twice, but only get one warning.
-            cov.switch_context("test1")                                 # pragma: nested
-            cov.switch_context("test2")                                 # pragma: nested
-        cov.stop()                                                      # pragma: nested
+        with cov.collect():
+            with pytest.warns(Warning) as warns:
+                # Switch twice, but only get one warning.
+                cov.switch_context("test1")
+                cov.switch_context("test2")
         assert_coverage_warnings(warns, "Conflicting dynamic contexts (dynamic-conflict)")
 
     def test_unknown_dynamic_context(self) -> None:
@@ -726,36 +734,12 @@ class ApiTest(CoverageTest):
         with pytest.raises(CoverageException, match=msg):
             cov.switch_context("test1")
 
-        cov.start()
-        cov.switch_context("test2")                                     # pragma: nested
+        with cov.collect():
+            cov.switch_context("test2")
 
-        cov.stop()                                                      # pragma: nested
         with pytest.raises(CoverageException, match=msg):
             cov.switch_context("test3")
 
-    def test_config_crash(self) -> None:
-        # The internal '[run] _crash' setting can be used to artificially raise
-        # exceptions from inside Coverage.
-        cov = coverage.Coverage()
-        cov.set_option("run:_crash", "test_config_crash")
-        with pytest.raises(Exception, match="Crashing because called by test_config_crash"):
-            cov.start()
-
-    def test_config_crash_no_crash(self) -> None:
-        # '[run] _crash' really checks the call stack.
-        cov = coverage.Coverage()
-        cov.set_option("run:_crash", "not_my_caller")
-        cov.start()
-        cov.stop()
-
-    def test_run_debug_sys(self) -> None:
-        # https://github.com/nedbat/coveragepy/issues/907
-        cov = coverage.Coverage()
-        cov.start()
-        d = dict(cov.sys_info())        # pragma: nested
-        cov.stop()                      # pragma: nested
-        assert cast(str, d['data_file']).endswith(".coverage")
-
 
 class CurrentInstanceTest(CoverageTest):
     """Tests of Coverage.current()."""
@@ -780,12 +764,10 @@ class CurrentInstanceTest(CoverageTest):
         self.assert_current_is_none(cur1)
         assert cur0 is cur1
         # Starting the instance makes it current.
-        cov.start()
-        if "# pragma: nested":
+        with cov.collect():
             cur2 = coverage.Coverage.current()
             assert cur2 is cov
             # Stopping the instance makes current None again.
-            cov.stop()
 
         cur3 = coverage.Coverage.current()
         self.assert_current_is_none(cur3)
@@ -815,19 +797,14 @@ class NamespaceModuleTest(UsingModulesMixin, CoverageTest):
             cov.report()
 
 
-class CoverageUsePkgs(Protocol):
-    """A number of test classes have the same helper method."""
-    def coverage_usepkgs(
-        self,                       # pylint: disable=unused-argument
-        **kwargs: TCovKwargs,
-    ) -> Iterable[str]:
-        """Run coverage on usepkgs, return a line summary. kwargs are for Coverage(**kwargs)."""
-        return ""
-
-
-class IncludeOmitTestsMixin(CoverageUsePkgs, UsingModulesMixin, CoverageTest):
+class IncludeOmitTestsMixin(UsingModulesMixin, CoverageTest):
     """Test methods for coverage methods taking include and omit."""
 
+    # An abstract method for subclasses to define, to appease mypy.
+    def coverage_usepkgs(self, **kwargs_unused: TCovKwargs) -> Iterable[str]:
+        """Run coverage on usepkgs, return a line summary. kwargs are for Coverage(**kwargs)."""
+        raise NotImplementedError()     # pragma: not covered
+
     def filenames_in(self, summary: Iterable[str], filenames: str) -> None:
         """Assert the `filenames` are in the `summary`."""
         for filename in filenames.split():
@@ -906,9 +883,8 @@ class SourceIncludeOmitTest(IncludeOmitTestsMixin, CoverageTest):
 
         """
         cov = coverage.Coverage(**kwargs)
-        cov.start()
-        import usepkgs  # pragma: nested   # pylint: disable=import-error, unused-import
-        cov.stop()      # pragma: nested
+        with cov.collect():
+            import usepkgs  # pylint: disable=import-error, unused-import
         with self.assert_warnings(cov, []):
             data = cov.get_data()
         summary = line_counts(data)
@@ -925,7 +901,7 @@ class SourceIncludeOmitTest(IncludeOmitTestsMixin, CoverageTest):
         cov = coverage.Coverage(source=["pkg1"], include=["pkg2"])
         with self.assert_warnings(cov, ["--include is ignored because --source is set"]):
             cov.start()
-        cov.stop()      # pragma: nested
+        cov.stop()
 
     def test_source_package_as_package(self) -> None:
         assert not os.path.isdir("pkg1")
@@ -999,9 +975,8 @@ class ReportIncludeOmitTest(IncludeOmitTestsMixin, CoverageTest):
     def coverage_usepkgs(self, **kwargs: TCovKwargs) -> Iterable[str]:
         """Try coverage.report()."""
         cov = coverage.Coverage()
-        cov.start()
-        import usepkgs  # pragma: nested   # pylint: disable=import-error, unused-import
-        cov.stop()      # pragma: nested
+        with cov.collect():
+            import usepkgs  # pylint: disable=import-error, unused-import
         report = io.StringIO()
         cov.report(file=report, **kwargs)
         return report.getvalue()
@@ -1018,9 +993,8 @@ class XmlIncludeOmitTest(IncludeOmitTestsMixin, CoverageTest):
     def coverage_usepkgs(self, **kwargs: TCovKwargs) -> Iterable[str]:
         """Try coverage.xml_report()."""
         cov = coverage.Coverage()
-        cov.start()
-        import usepkgs  # pragma: nested   # pylint: disable=import-error, unused-import
-        cov.stop()      # pragma: nested
+        with cov.collect():
+            import usepkgs  # pylint: disable=import-error, unused-import
         cov.xml_report(outfile="-", **kwargs)
         return self.stdout()
 
@@ -1489,146 +1463,3 @@ class CombiningTest(CoverageTest):
         # After combining, the .coverage file & the original combined file should still be there.
         self.assert_exists(".coverage")
         self.assert_file_count(".coverage.*", 2)
-
-
-class ReportMapsPathsTest(CoverageTest):
-    """Check that reporting implicitly maps paths."""
-
-    def make_files(self, data: str, settings: bool = False) -> None:
-        """Create the test files we need for line coverage."""
-        src = """\
-            if VER == 1:
-                print("line 2")
-            if VER == 2:
-                print("line 4")
-            if VER == 3:
-                print("line 6")
-            """
-        self.make_file("src/program.py", src)
-        self.make_file("ver1/program.py", src)
-        self.make_file("ver2/program.py", src)
-
-        if data == "line":
-            self.make_data_file(
-                lines={
-                    abs_file("ver1/program.py"): [1, 2, 3, 5],
-                    abs_file("ver2/program.py"): [1, 3, 4, 5],
-                }
-            )
-        else:
-            self.make_data_file(
-                arcs={
-                    abs_file("ver1/program.py"): arcz_to_arcs(".1 12 23 35 5."),
-                    abs_file("ver2/program.py"): arcz_to_arcs(".1 13 34 45 5."),
-                }
-            )
-
-        if settings:
-            self.make_file(".coveragerc", """\
-                [paths]
-                source =
-                    src
-                    ver1
-                    ver2
-                """)
-
-    def test_map_paths_during_line_report_without_setting(self) -> None:
-        self.make_files(data="line")
-        cov = coverage.Coverage()
-        cov.load()
-        cov.report(show_missing=True)
-        expected = textwrap.dedent(os_sep("""\
-            Name              Stmts   Miss  Cover   Missing
-            -----------------------------------------------
-            ver1/program.py       6      2    67%   4, 6
-            ver2/program.py       6      2    67%   2, 6
-            -----------------------------------------------
-            TOTAL                12      4    67%
-            """))
-        assert expected == self.stdout()
-
-    def test_map_paths_during_line_report(self) -> None:
-        self.make_files(data="line", settings=True)
-        cov = coverage.Coverage()
-        cov.load()
-        cov.report(show_missing=True)
-        expected = textwrap.dedent(os_sep("""\
-            Name             Stmts   Miss  Cover   Missing
-            ----------------------------------------------
-            src/program.py       6      1    83%   6
-            ----------------------------------------------
-            TOTAL                6      1    83%
-            """))
-        assert expected == self.stdout()
-
-    def test_map_paths_during_branch_report_without_setting(self) -> None:
-        self.make_files(data="arcs")
-        cov = coverage.Coverage(branch=True)
-        cov.load()
-        cov.report(show_missing=True)
-        expected = textwrap.dedent(os_sep("""\
-            Name              Stmts   Miss Branch BrPart  Cover   Missing
-            -------------------------------------------------------------
-            ver1/program.py       6      2      6      3    58%   1->3, 4, 6
-            ver2/program.py       6      2      6      3    58%   2, 3->5, 6
-            -------------------------------------------------------------
-            TOTAL                12      4     12      6    58%
-            """))
-        assert expected == self.stdout()
-
-    def test_map_paths_during_branch_report(self) -> None:
-        self.make_files(data="arcs", settings=True)
-        cov = coverage.Coverage(branch=True)
-        cov.load()
-        cov.report(show_missing=True)
-        expected = textwrap.dedent(os_sep("""\
-            Name             Stmts   Miss Branch BrPart  Cover   Missing
-            ------------------------------------------------------------
-            src/program.py       6      1      6      1    83%   6
-            ------------------------------------------------------------
-            TOTAL                6      1      6      1    83%
-            """))
-        assert expected == self.stdout()
-
-    def test_map_paths_during_annotate(self) -> None:
-        self.make_files(data="line", settings=True)
-        cov = coverage.Coverage()
-        cov.load()
-        cov.annotate()
-        self.assert_exists(os_sep("src/program.py,cover"))
-        self.assert_doesnt_exist(os_sep("ver1/program.py,cover"))
-        self.assert_doesnt_exist(os_sep("ver2/program.py,cover"))
-
-    def test_map_paths_during_html_report(self) -> None:
-        self.make_files(data="line", settings=True)
-        cov = coverage.Coverage()
-        cov.load()
-        cov.html_report()
-        contains("htmlcov/index.html", os_sep("src/program.py"))
-        doesnt_contain("htmlcov/index.html", os_sep("ver1/program.py"), os_sep("ver2/program.py"))
-
-    def test_map_paths_during_xml_report(self) -> None:
-        self.make_files(data="line", settings=True)
-        cov = coverage.Coverage()
-        cov.load()
-        cov.xml_report()
-        contains("coverage.xml", "src/program.py")
-        doesnt_contain("coverage.xml", "ver1/program.py", "ver2/program.py")
-
-    def test_map_paths_during_json_report(self) -> None:
-        self.make_files(data="line", settings=True)
-        cov = coverage.Coverage()
-        cov.load()
-        cov.json_report()
-        def os_sepj(s: str) -> str:
-            return os_sep(s).replace("\\", r"\\")
-        contains("coverage.json", os_sepj("src/program.py"))
-        doesnt_contain("coverage.json", os_sepj("ver1/program.py"), os_sepj("ver2/program.py"))
-
-    def test_map_paths_during_lcov_report(self) -> None:
-        self.make_files(data="line", settings=True)
-        cov = coverage.Coverage()
-        cov.load()
-        cov.lcov_report()
-        contains("coverage.lcov", os_sep("src/program.py"))
-        doesnt_contain("coverage.lcov", os_sep("ver1/program.py"), os_sep("ver2/program.py"))
