@@ -24,7 +24,6 @@ xfail_pypy_3882 = pytest.mark.xfail(
     reason="https://foss.heptapod.net/pypy/pypy/-/issues/3882",
 )
 
-
 class SimpleArcTest(CoverageTest):
     """Tests for coverage.py's arc measurement."""
 
@@ -33,13 +32,15 @@ class SimpleArcTest(CoverageTest):
             a = 1
             b = 2
             """,
-            arcz=".1 12 2.")
+            arcz=".1 12 2."
+        )
         self.check_coverage("""\
             a = 1
 
             b = 3
             """,
-            arcz=".1 13 3.")
+            arcz=".1 13 3."
+        )
         line1 = 1 if env.PYBEHAVIOR.module_firstline_1 else 2
         self.check_coverage("""\
 
@@ -58,7 +59,8 @@ class SimpleArcTest(CoverageTest):
 
             foo()
             """,
-            arcz=".1 .2 14 2. 4.")
+            arcz=".1 .2 14 2. 4."
+        )
 
     def test_if(self) -> None:
         self.check_coverage("""\
@@ -67,14 +69,16 @@ class SimpleArcTest(CoverageTest):
                 a = 3
             assert a == 3
             """,
-            arcz=".1 12 23 24 34 4.", arcz_missing="24")
+            arcz=".1 12 23 24 34 4.", arcz_missing="24"
+        )
         self.check_coverage("""\
             a = 1
             if len([]) == 1:
                 a = 3
             assert a == 1
             """,
-            arcz=".1 12 23 24 34 4.", arcz_missing="23 34")
+            arcz=".1 12 23 24 34 4.", arcz_missing="23 34"
+        )
 
     def test_if_else(self) -> None:
         self.check_coverage("""\
@@ -84,7 +88,8 @@ class SimpleArcTest(CoverageTest):
                 a = 4
             assert a == 2
             """,
-            arcz=".1 12 25 14 45 5.", arcz_missing="14 45")
+            arcz=".1 12 25 14 45 5.", arcz_missing="14 45"
+        )
         self.check_coverage("""\
             if len([]) == 1:
                 a = 2
@@ -92,7 +97,8 @@ class SimpleArcTest(CoverageTest):
                 a = 4
             assert a == 4
             """,
-            arcz=".1 12 25 14 45 5.", arcz_missing="12 25")
+            arcz=".1 12 25 14 45 5.", arcz_missing="12 25"
+        )
 
     def test_compact_if(self) -> None:
         self.check_coverage("""\
@@ -109,7 +115,8 @@ class SimpleArcTest(CoverageTest):
             a = fn(1)
             assert a is True
             """,
-            arcz=".1 14 45 5.  .2 2. 23 3.", arcz_missing="23 3.")
+            arcz=".1 14 45 5.  .2 2. 23 3.", arcz_missing="23 3."
+        )
 
     def test_multiline(self) -> None:
         self.check_coverage("""\
@@ -356,7 +363,8 @@ class LoopArcTest(CoverageTest):
                 a = i
             assert a == -1
             """,
-            arcz=".1 12 23 32 24 4.", arcz_missing="23 32")
+            arcz=".1 12 23 32 24 4.", arcz_missing="23 32"
+        )
 
     def test_nested_loop(self) -> None:
         self.check_coverage("""\
@@ -414,16 +422,15 @@ class LoopArcTest(CoverageTest):
                     break
             assert a == 2 and i == 2    # 7
             """,
-            arcz=".1 12 23 34 45 25 56 51 67 17 7.", arcz_missing="17 25")
+            arcz=".1 12 23 34 45 25 56 51 67 17 7.", arcz_missing="17 25"
+        )
 
     def test_while_1(self) -> None:
         # With "while 1", the loop knows it's constant.
         if env.PYBEHAVIOR.keep_constant_test:
             arcz = ".1 12 23 34 45 36 62 57 7."
-        elif env.PYBEHAVIOR.nix_while_true:
-            arcz = ".1 13 34 45 36 63 57 7."
         else:
-            arcz = ".1 12 23 34 45 36 63 57 7."
+            arcz = ".1 13 34 45 36 63 57 7."
         self.check_coverage("""\
             a, i = 1, 0
             while 1:
@@ -441,10 +448,8 @@ class LoopArcTest(CoverageTest):
         # 3.x thinks it's constant.
         if env.PYBEHAVIOR.keep_constant_test:
             arcz = ".1 12 23 34 45 36 62 57 7."
-        elif env.PYBEHAVIOR.nix_while_true:
-            arcz = ".1 13 34 45 36 63 57 7."
         else:
-            arcz = ".1 12 23 34 45 36 63 57 7."
+            arcz = ".1 13 34 45 36 63 57 7."
         self.check_coverage("""\
             a, i = 1, 0
             while True:
@@ -470,10 +475,8 @@ class LoopArcTest(CoverageTest):
         assert self.stdout() == 'done\n'
         if env.PYBEHAVIOR.keep_constant_test:
             num_stmts = 3
-        elif env.PYBEHAVIOR.nix_while_true:
-            num_stmts = 2
         else:
-            num_stmts = 3
+            num_stmts = 2
         expected = "zero.py {n} {n} 0 0 0% 1-3".format(n=num_stmts)
         report = self.get_report(cov, show_missing=True)
         squeezed = self.squeezed_lines(report)
@@ -484,10 +487,8 @@ class LoopArcTest(CoverageTest):
         # A continue in a while-true needs to jump to the right place.
         if env.PYBEHAVIOR.keep_constant_test:
             arcz = ".1 12 23 34 45 52 46 67 7."
-        elif env.PYBEHAVIOR.nix_while_true:
-            arcz = ".1 13 34 45 53 46 67 7."
         else:
-            arcz = ".1 12 23 34 45 53 46 67 7."
+            arcz = ".1 13 34 45 53 46 67 7."
         self.check_coverage("""\
             up = iter('ta')
             while True:
@@ -559,6 +560,10 @@ class LoopArcTest(CoverageTest):
         )
 
     def test_confusing_for_loop_bug_175(self) -> None:
+        if env.PYBEHAVIOR.comprehensions_are_functions:
+            extra_arcz = " -22 2-2"
+        else:
+            extra_arcz = ""
         self.check_coverage("""\
             o = [(1,2), (3,4)]
             o = [a for a in o]
@@ -566,7 +571,7 @@ class LoopArcTest(CoverageTest):
                 x = tup[0]
                 y = tup[1]
             """,
-            arcz=".1 -22 2-2 12 23 34 45 53 3.",
+            arcz=".1 12 23 34 45 53 3." + extra_arcz,
         )
         self.check_coverage("""\
             o = [(1,2), (3,4)]
@@ -574,7 +579,7 @@ class LoopArcTest(CoverageTest):
                 x = tup[0]
                 y = tup[1]
             """,
-            arcz=".1 12 -22 2-2 23 34 42 2.",
+            arcz=".1 12 23 34 42 2." + extra_arcz,
         )
 
     # https://bugs.python.org/issue44672
@@ -639,6 +644,10 @@ class LoopArcTest(CoverageTest):
         )
 
     def test_other_comprehensions(self) -> None:
+        if env.PYBEHAVIOR.comprehensions_are_functions:
+            extra_arcz = " -22 2-2"
+        else:
+            extra_arcz = ""
         # Set comprehension:
         self.check_coverage("""\
             o = ((1,2), (3,4))
@@ -647,7 +656,7 @@ class LoopArcTest(CoverageTest):
                 x = tup[0]
                 y = tup[1]
             """,
-            arcz=".1 -22 2-2 12 23 34 45 53 3.",
+            arcz=".1 12 23 34 45 53 3." + extra_arcz,
         )
         # Dict comprehension:
         self.check_coverage("""\
@@ -657,10 +666,14 @@ class LoopArcTest(CoverageTest):
                 x = tup[0]
                 y = tup[1]
             """,
-            arcz=".1 -22 2-2 12 23 34 45 53 3.",
+            arcz=".1 12 23 34 45 53 3." + extra_arcz,
         )
 
     def test_multiline_dict_comp(self) -> None:
+        if env.PYBEHAVIOR.comprehensions_are_functions:
+            extra_arcz = " 2-2"
+        else:
+            extra_arcz = ""
         # Multiline dict comp:
         self.check_coverage("""\
             # comment
@@ -675,7 +688,7 @@ class LoopArcTest(CoverageTest):
             }
             x = 11
             """,
-            arcz="-22 2B B-2   2-2"
+            arcz="-22 2B B-2" + extra_arcz,
         )
         # Multi dict comp:
         self.check_coverage("""\
@@ -695,7 +708,7 @@ class LoopArcTest(CoverageTest):
             }
             x = 15
             """,
-            arcz="-22 2F F-2   2-2"
+            arcz="-22 2F F-2" + extra_arcz,
         )
 
 
@@ -711,7 +724,8 @@ class ExceptionArcTest(CoverageTest):
                 b = 5
             assert a == 3 and b == 1
             """,
-            arcz=".1 12 23 36 45 56 6.", arcz_missing="45 56")
+            arcz=".1 12 23 36 45 56 6.", arcz_missing="45 56"
+        )
 
     def test_raise_followed_by_statement(self) -> None:
         if env.PYBEHAVIOR.omit_after_jump:
@@ -972,7 +986,8 @@ class ExceptionArcTest(CoverageTest):
                 c = 7
             assert a == 3 and b == 1 and c == 7
             """,
-            arcz=".1 12 23 45 37 57 78 8.", arcz_missing="45 57")
+            arcz=".1 12 23 45 37 57 78 8.", arcz_missing="45 57"
+        )
         self.check_coverage("""\
             a, b, c = 1, 1, 1
             def oops(x):
@@ -988,7 +1003,8 @@ class ExceptionArcTest(CoverageTest):
             assert a == 5 and b == 9 and c == 11
             """,
             arcz=".1 12 -23 3-2 24 45 56 67 7B 89 9B BC C.",
-            arcz_missing="67 7B", arcz_unpredicted="68")
+            arcz_missing="67 7B", arcz_unpredicted="68"
+        )
 
     def test_multiple_except_clauses(self) -> None:
         self.check_coverage("""\
@@ -1218,7 +1234,6 @@ class YieldTest(CoverageTest):
             """,
             arcz=".1 15 56 65 5.  .2 23 32 2.  -33 3-3",
         )
-
         self.check_coverage("""\
             def run():
                 yield lambda: 100
@@ -1230,7 +1245,6 @@ class YieldTest(CoverageTest):
             """,
             arcz=".1 16 67 76 6.  .2 23 34 43 3.   -22 2-2  -44 4-4",
         )
-
         self.check_coverage("""\
             def run():
                 yield lambda: 100  # no branch miss
@@ -1297,9 +1311,9 @@ class YieldTest(CoverageTest):
         # https://github.com/nedbat/coveragepy/issues/440
         self.check_coverage("""\
             def gen():
-                print("yup")
-                yield "yielded"
-                print("nope")
+                print(2)
+                yield 3
+                print(4)
 
             print(next(gen()))
             """,
@@ -1308,6 +1322,7 @@ class YieldTest(CoverageTest):
             arcz=".1 16 6.  .2 23 34 4.",
             arcz_missing="34 4.",
         )
+        assert self.stdout() == "2\n3\n"
 
 
 @pytest.mark.skipif(not env.PYBEHAVIOR.match_case, reason="Match-case is new in 3.10")
@@ -1719,9 +1734,7 @@ class DecoratorArcTest(CoverageTest):
     def test_bug_466a(self) -> None:
         # A bad interaction between decorators and multi-line list assignments,
         # believe it or not...!
-        arcz = ".1 1A A.  13 3.     -35 58 8-3 "
-        if env.PYBEHAVIOR.trace_decorated_def:
-            arcz = arcz.replace("3.", "34 4.")
+        arcz = ".1 1A A.  13 34 4.  -35 58 8-3 "
         if env.PYBEHAVIOR.trace_decorator_line_again:
             arcz += "43 "
         # This example makes more sense when considered in tandem with 466b below.
@@ -1743,9 +1756,7 @@ class DecoratorArcTest(CoverageTest):
     def test_bug_466b(self) -> None:
         # A bad interaction between decorators and multi-line list assignments,
         # believe it or not...!
-        arcz = ".1 1A A.  13 3.     -35 58 8-3 "
-        if env.PYBEHAVIOR.trace_decorated_def:
-            arcz = arcz.replace("3.", "34 4.")
+        arcz = ".1 1A A.  13 34 4.  -35 58 8-3 "
         if env.PYBEHAVIOR.trace_decorator_line_again:
             arcz += "43 "
         self.check_coverage("""\
@@ -1932,9 +1943,7 @@ class AsyncTest(CoverageTest):
         )
 
     def test_async_decorator(self) -> None:
-        arcz = ".1 14 4.     .2 2.  -46 6-4 "
-        if env.PYBEHAVIOR.trace_decorated_def:
-            arcz = arcz.replace("4.", "45 5.")
+        arcz = ".1 14 45 5.  .2 2.  -46 6-4 "
         if env.PYBEHAVIOR.trace_decorator_line_again:
             arcz += "54 "
         self.check_coverage("""\
