@@ -6,6 +6,7 @@
 from __future__ import annotations
 
 import ast
+import os
 import pprint
 import re
 import sys
@@ -333,16 +334,25 @@ class CmdLineTest(BaseCmdLineTest):
     def test_debug_premain(self) -> None:
         self.command_line("debug premain")
         out = self.stdout()
+        # -- premain ---------------------------------------------------
         #   ... many lines ...
+        #           _multicall : /Users/ned/cov/trunk/.tox/py39/site-packages/pluggy/_callers.py:77
         #   pytest_pyfunc_call : /Users/ned/cov/trunk/.tox/py39/site-packages/_pytest/python.py:183
         #   test_debug_premain : /Users/ned/cov/trunk/tests/test_cmdline.py:284
         #         command_line : /Users/ned/cov/trunk/tests/coveragetest.py:309
         #         command_line : /Users/ned/cov/trunk/tests/coveragetest.py:472
         #         command_line : /Users/ned/cov/trunk/coverage/cmdline.py:592
         #             do_debug : /Users/ned/cov/trunk/coverage/cmdline.py:804
-        assert re.search(r"(?m)^\s+test_debug_premain : .*[/\\]tests[/\\]test_cmdline.py:\d+$", out)
-        assert re.search(r"(?m)^\s+command_line : .*[/\\]coverage[/\\]cmdline.py:\d+$", out)
-        assert re.search(r"(?m)^\s+do_debug : .*[/\\]coverage[/\\]cmdline.py:\d+$", out)
+        lines = out.splitlines()
+        s = re.escape(os.sep)
+        assert lines[0].startswith("-- premain ----")
+        assert len(lines) > 25
+        assert re.search(fr"{s}site-packages{s}_pytest{s}", out)
+        assert re.search(fr"{s}site-packages{s}pluggy{s}", out)
+        assert re.search(fr"(?m)^\s+test_debug_premain : .*{s}tests{s}test_cmdline.py:\d+$", out)
+        assert re.search(fr"(?m)^\s+command_line : .*{s}coverage{s}cmdline.py:\d+$", out)
+        assert re.search(fr"(?m)^\s+do_debug : .*{s}coverage{s}cmdline.py:\d+$", out)
+        assert "do_debug : " in lines[-1]
 
     def test_erase(self) -> None:
         # coverage erase
@@ -1081,7 +1091,7 @@ class CmdMainTest(CoverageTest):
                 print("Hello, world!")
             elif argv[0] == 'raise':
                 try:
-                    raise Exception("oh noes!")
+                    raise RuntimeError("oh noes!")
                 except:
                     raise _ExceptionDuringRun(*sys.exc_info()) from None
             elif argv[0] == 'internalraise':
@@ -1111,8 +1121,8 @@ class CmdMainTest(CoverageTest):
         print(err)
         err_parts = err.splitlines(keepends=True)
         assert err_parts[0] == 'Traceback (most recent call last):\n'
-        assert '    raise Exception("oh noes!")\n' in err_parts
-        assert err_parts[-1] == 'Exception: oh noes!\n'
+        assert '    raise RuntimeError("oh noes!")\n' in err_parts
+        assert err_parts[-1] == 'RuntimeError: oh noes!\n'
 
     def test_internalraise(self) -> None:
         with pytest.raises(ValueError, match="coverage is broken"):
@@ -1214,6 +1224,7 @@ class FailUnderTest(CoverageTest):
             "Coverage failure: total of 20.5 is less than fail-under=20.6\n"),
         (20.12345, "report --fail-under=20.1235 --precision=5", 2,
             "Coverage failure: total of 20.12345 is less than fail-under=20.12350\n"),
+        (20.12339, "report --fail-under=20.1234 --precision=4", 0, ""),
     ])
     def test_fail_under_with_precision(self, result: float, cmd: str, ret: int, msg: str) -> None:
         cov = CoverageReportingFake(report_result=result)
