@@ -18,12 +18,15 @@ import pytest
 import coverage
 from coverage import env
 from coverage.debug import (
-    DebugOutputFile,
-    clipped_repr, filter_text, info_formatter, info_header, short_id, short_stack,
+    DebugControl, DebugOutputFile,
+    auto_repr, clipped_repr, exc_one_line, filter_text,
+    info_formatter, info_header,
+    relevant_environment_display, short_id, short_filename, short_stack,
 )
+from coverage.exceptions import DataError
 
 from tests.coveragetest import CoverageTest
-from tests.helpers import re_line, re_lines, re_lines_text
+from tests.helpers import DebugControlString, re_line, re_lines, re_lines_text
 
 
 class InfoFormatterTest(CoverageTest):
@@ -68,7 +71,7 @@ class InfoFormatterTest(CoverageTest):
     ("hello there",     "-- hello there -----------------------------------------------"),
 ])
 def test_info_header(label: str, header: str) -> None:
-    assert info_header(label) == header
+    assert header == info_header(label)
 
 
 @pytest.mark.parametrize("id64, id16", [
@@ -78,7 +81,7 @@ def test_info_header(label: str, header: str) -> None:
     (0x1234cba956780fed, 0x8008),
 ])
 def test_short_id(id64: int, id16: int) -> None:
-    assert short_id(id64) == id16
+    assert id16 == short_id(id64)
 
 
 @pytest.mark.parametrize("text, numchars, result", [
@@ -86,7 +89,7 @@ def test_short_id(id64: int, id16: int) -> None:
     ("0123456789abcdefghijklmnopqrstuvwxyz", 15, "'01234...vwxyz'"),
 ])
 def test_clipped_repr(text: str, numchars: int, result: str) -> None:
-    assert clipped_repr(text, numchars) == result
+    assert result == clipped_repr(text, numchars)
 
 
 @pytest.mark.parametrize("text, filters, result", [
@@ -101,7 +104,7 @@ def test_filter_text(
     filters: Iterable[Callable[[str], str]],
     result: str,
 ) -> None:
-    assert filter_text(text, filters) == result
+    assert result == filter_text(text, filters)
 
 
 class DebugTraceTest(CoverageTest):
@@ -207,6 +210,15 @@ class DebugTraceTest(CoverageTest):
         vtuple = ast.literal_eval(pyversion.partition(":")[-1].strip())
         assert vtuple[:5] == sys.version_info
 
+    def test_debug_process(self) -> None:
+        out_text = self.f1_debug_output(["trace", "process"])
+        assert f"New process: pid={os.getpid()}, executable:" in out_text
+
+    def test_debug_pytest(self) -> None:
+        out_text = self.f1_debug_output(["trace", "pytest"])
+        ctx = "tests/test_debug.py::DebugTraceTest::test_debug_pytest (call)"
+        assert f"Pytest context: {ctx}" in out_text
+
 
 def assert_good_debug_sys(out_text: str) -> None:
     """Assert that `str` is good output for debug=sys."""
@@ -240,20 +252,20 @@ class DebugOutputTest(CoverageTest):
     def test_stderr_default(self) -> None:
         self.debug_sys()
         out, err = self.stdouterr()
-        assert out == ""
+        assert "" == out
         assert_good_debug_sys(err)
 
     def test_envvar(self) -> None:
         self.set_environ("COVERAGE_DEBUG_FILE", "debug.out")
         self.debug_sys()
-        assert self.stdouterr() == ("", "")
+        assert ("", "") == self.stdouterr()
         with open("debug.out") as f:
             assert_good_debug_sys(f.read())
 
     def test_config_file(self) -> None:
         self.make_file(".coveragerc", "[run]\ndebug_file = lotsa_info.txt")
         self.debug_sys()
-        assert self.stdouterr() == ("", "")
+        assert ("", "") == self.stdouterr()
         with open("lotsa_info.txt") as f:
             assert_good_debug_sys(f.read())
 
@@ -261,10 +273,59 @@ class DebugOutputTest(CoverageTest):
         self.set_environ("COVERAGE_DEBUG_FILE", "stdout")
         self.debug_sys()
         out, err = self.stdouterr()
-        assert err == ""
+        assert "" == err
         assert_good_debug_sys(out)
 
 
+class DebugControlTest(CoverageTest):
+    """Tests of DebugControl (via DebugControlString)."""
+
+    run_in_temp_dir = False
+
+    def test_debug_control(self) -> None:
+        debug = DebugControlString(["yes"])
+        assert debug.should("yes")
+        debug.write("YES")
+        assert not debug.should("no")
+        assert "YES\n" == debug.get_output()
+
+    def test_debug_write_exceptions(self) -> None:
+        debug = DebugControlString(["yes"])
+        try:
+            raise RuntimeError('Oops') # This is in the traceback
+        except Exception as exc:
+            debug.write("Something happened", exc=exc)
+        lines = debug.get_output().splitlines()
+        assert "Something happened" == lines[0]
+        assert "Traceback (most recent call last):" == lines[1]
+        assert "    raise RuntimeError('Oops') # This is in the traceback" in lines
+        assert "RuntimeError: Oops" == lines[-1]
+
+    def test_debug_write_self(self) -> None:
+        class DebugWritingClass:
+            """A simple class to show 'self:' debug messages."""
+            def __init__(self, debug: DebugControl) -> None:
+                # This line will have "self:" reported.
+                debug.write("Hello from me")
+
+            def __repr__(self) -> str:
+                return "<<DebugWritingClass object!>>"
+
+        def run_some(debug: DebugControl) -> None:
+            # This line will have no "self:" because there's no local self.
+            debug.write("In run_some")
+            DebugWritingClass(debug)
+
+        debug = DebugControlString(["self"])
+        run_some(debug)
+        lines = debug.get_output().splitlines()
+        assert lines == [
+            "In run_some",
+            "Hello from me",
+            "self: <<DebugWritingClass object!>>",
+        ]
+
+
 def f_one(*args: Any, **kwargs: Any) -> str:
     """First of the chain of functions for testing `short_stack`."""
     return f_two(*args, **kwargs)
@@ -285,15 +346,106 @@ class ShortStackTest(CoverageTest):
 
     def test_short_stack(self) -> None:
         stack = f_one().splitlines()
-        assert len(stack) > 10
-        assert "f_three" in stack[-1]
-        assert "f_two" in stack[-2]
-        assert "f_one" in stack[-3]
-
-    def test_short_stack_limit(self) -> None:
-        stack = f_one(limit=5).splitlines()
-        assert len(stack) == 5
+        assert 4 == len(stack)
+        assert "test_short_stack" in stack[0]
+        assert "f_one" in stack[1]
+        assert "f_two" in stack[2]
+        assert "f_three" in stack[3]
 
     def test_short_stack_skip(self) -> None:
         stack = f_one(skip=1).splitlines()
-        assert "f_two" in stack[-1]
+        assert 3 == len(stack)
+        assert "test_short_stack" in stack[0]
+        assert "f_one" in stack[1]
+        assert "f_two" in stack[2]
+
+    def test_short_stack_full(self) -> None:
+        stack_text = f_one(full=True)
+        s = re.escape(os.sep)
+        if env.WINDOWS:
+            pylib = "[Ll]ib"
+        else:
+            py = "pypy" if env.PYPY else "python"
+            majv, minv = sys.version_info[:2]
+            pylib = f"lib{s}{py}{majv}.{minv}"
+        assert len(re_lines(fr"{s}{pylib}{s}site-packages{s}_pytest", stack_text)) > 3
+        assert len(re_lines(fr"{s}{pylib}{s}site-packages{s}pluggy", stack_text)) > 3
+        assert not re_lines(r" 0x[0-9a-fA-F]+", stack_text) # No frame ids
+        stack = stack_text.splitlines()
+        assert len(stack) > 25
+        assert "test_short_stack" in stack[-4]
+        assert "f_one" in stack[-3]
+        assert "f_two" in stack[-2]
+        assert "f_three" in stack[-1]
+
+    def test_short_stack_short_filenames(self) -> None:
+        stack_text = f_one(full=True, short_filenames=True)
+        s = re.escape(os.sep)
+        assert not re_lines(r"site-packages", stack_text)
+        assert len(re_lines(fr"syspath:{s}_pytest", stack_text)) > 3
+        assert len(re_lines(fr"syspath:{s}pluggy", stack_text)) > 3
+
+    def test_short_stack_frame_ids(self) -> None:
+        stack = f_one(full=True, frame_ids=True).splitlines()
+        assert len(stack) > 25
+        frame_ids = [m[0] for line in stack if (m := re.search(r" 0x[0-9a-fA-F]{6,}", line))]
+        # Every line has a frame id.
+        assert len(frame_ids) == len(stack)
+        # All the frame ids are different.
+        assert len(set(frame_ids)) == len(frame_ids)
+
+
+class ShortFilenameTest(CoverageTest):
+    """Tests of debug.py:short_filename."""
+
+    def test_short_filename(self) -> None:
+        s = os.sep
+        se = re.escape(s)
+        assert short_filename(ast.__file__) == f"syspath:{s}ast.py"
+        assert short_filename(pytest.__file__) == f"syspath:{s}pytest{s}__init__.py"
+        assert short_filename(env.__file__) == f"cov:{s}env.py"
+        self.make_file("hello.txt", "hi")
+        short_hello = short_filename(os.path.abspath("hello.txt"))
+        assert re.match(fr"tmp:{se}t\d+{se}hello.txt", short_hello)
+        oddball = f"{s}xyzzy{s}plugh{s}foo.txt"
+        assert short_filename(oddball) == oddball
+        assert short_filename(None) is None
+
+
+def test_relevant_environment_display() -> None:
+    env_vars = {
+        "HOME": "my home",
+        "HOME_DIR": "other place",
+        "XYZ_NEVER_MIND": "doesn't matter",
+        "SOME_PYOTHER": "xyz123",
+        "COVERAGE_THING": "abcd",
+        "MY_PYPI_TOKEN": "secret.something",
+        "TMP": "temporary",
+    }
+    expected = [
+        ("COVERAGE_THING", "abcd"),
+        ("HOME", "my home"),
+        ("MY_PYPI_TOKEN", "******.*********"),
+        ("SOME_PYOTHER", "xyz123"),
+        ("TMP", "temporary"),
+    ]
+    assert expected == relevant_environment_display(env_vars)
+
+
+def test_exc_one_line() -> None:
+    try:
+        raise DataError("wtf?")
+    except Exception as exc:
+        assert "coverage.exceptions.DataError: wtf?" == exc_one_line(exc)
+
+
+def test_auto_repr() -> None:
+    class MyStuff:
+        """Random class to test auto_repr."""
+        def __init__(self) -> None:
+            self.x = 17
+            self.y = "hello"
+        __repr__ = auto_repr
+    stuff = MyStuff()
+    setattr(stuff, "$coverage.object_id", 123456)
+    assert re.match(r"<MyStuff @0x[a-f\d]+ x=17 y='hello'>", repr(stuff))
