@@ -5,6 +5,7 @@
 
 from __future__ import annotations
 
+import csv
 import glob
 import os
 import os.path
@@ -14,6 +15,7 @@ import stat
 import sys
 import textwrap
 
+from pathlib import Path
 from typing import Any
 
 import pytest
@@ -24,7 +26,7 @@ from coverage.data import line_counts
 from coverage.files import abs_file, python_reported_file
 
 from tests.coveragetest import CoverageTest, TESTS_DIR
-from tests.helpers import re_lines_text
+from tests.helpers import re_lines, re_lines_text
 
 
 class ProcessTest(CoverageTest):
@@ -50,6 +52,7 @@ class ProcessTest(CoverageTest):
             """)
 
         self.assert_doesnt_exist(".coverage")
+        self.add_test_modules_to_pythonpath()
         out = self.run_command("coverage run mycode.py")
         self.assert_exists(".coverage")
         assert out == 'done\n'
@@ -58,7 +61,7 @@ class ProcessTest(CoverageTest):
         # Test that we are setting COVERAGE_RUN when we run.
         self.make_file("envornot.py", """\
             import os
-            print(os.environ.get("COVERAGE_RUN", "nope"))
+            print(os.getenv("COVERAGE_RUN", "nope"))
             """)
         self.del_environ("COVERAGE_RUN")
         # Regular Python doesn't have the environment variable.
@@ -363,44 +366,59 @@ class ProcessTest(CoverageTest):
         self.make_file("fork.py", """\
             import os
 
-            def child():
-                print('Child!')
+            print(f"parent,{os.getpid()}", flush=True)
+            ret = os.fork()
 
-            def main():
-                ret = os.fork()
-
-                if ret == 0:
-                    child()
-                else:
-                    os.waitpid(ret, 0)
-
-            main()
+            if ret == 0:
+                print(f"child,{os.getpid()}", flush=True)
+            else:
+                os.waitpid(ret, 0)
             """)
+        total_lines = 6
 
-        out = self.run_command("coverage run -p fork.py")
-        assert out == 'Child!\n'
+        self.set_environ("COVERAGE_DEBUG_FILE", "debug.out")
+        out = self.run_command("coverage run --debug=pid,process,trace -p fork.py")
+        pids = {key:int(pid) for key, pid in csv.reader(out.splitlines())}
+        assert set(pids) == {"parent", "child"}
         self.assert_doesnt_exist(".coverage")
 
         # After running the forking program, there should be two
-        # .coverage.machine.123 files.
+        # .coverage.machine.pid.randomword files.  The pids should match our
+        # processes, and the files should have different random words at the
+        # end of the file name.
         self.assert_file_count(".coverage.*", 2)
-
-        # The two data files should have different random numbers at the end of
-        # the file name.
         data_files = glob.glob(".coverage.*")
-        nums = {name.rpartition(".")[-1] for name in data_files}
-        assert len(nums) == 2, f"Same random: {data_files}"
+        filepids = {int(name.split(".")[-2]) for name in data_files}
+        assert filepids == set(pids.values())
+        suffixes = {name.split(".")[-1] for name in data_files}
+        assert len(suffixes) == 2, f"Same random suffix: {data_files}"
 
-        # Combine the parallel coverage data files into .coverage .
+        # Each data file should have a subset of the lines.
+        for data_file in data_files:
+            data = coverage.CoverageData(data_file)
+            data.read()
+            assert line_counts(data)["fork.py"] < total_lines
+
+        # Combine the parallel coverage data files into a .coverage file.
+        # After combining, there should be only the .coverage file.
         self.run_command("coverage combine")
         self.assert_exists(".coverage")
-
-        # After combining, there should be only the .coverage file.
         self.assert_file_count(".coverage.*", 0)
 
         data = coverage.CoverageData()
         data.read()
-        assert line_counts(data)['fork.py'] == 9
+        assert line_counts(data)["fork.py"] == total_lines
+
+        debug_text = Path("debug.out").read_text()
+        ppid = pids["parent"]
+        cpid = pids["child"]
+        assert ppid != cpid
+        plines = re_lines(fr"{ppid}\.[0-9a-f]+: New process: pid={ppid}, executable", debug_text)
+        assert len(plines) == 1
+        clines = re_lines(fr"{cpid}\.[0-9a-f]+: New process: forked {ppid} -> {cpid}", debug_text)
+        assert len(clines) == 1
+        reported_pids = {line.split(".")[0] for line in debug_text.splitlines()}
+        assert len(reported_pids) == 2
 
     def test_warnings_during_reporting(self) -> None:
         # While fixing issue #224, the warnings were being printed far too
@@ -492,24 +510,21 @@ class ProcessTest(CoverageTest):
             # Show the current frame's trace function, so that we can test what the
             # command-line options do to the trace function used.
 
-            import sys
+            import inspect
 
             # Show what the trace function is.  If a C-based function is used, then f_trace
             # may be None.
-            trace_fn = sys._getframe(0).f_trace
+            trace_fn = inspect.currentframe().f_trace
             if trace_fn is None:
                 trace_name = "None"
             else:
-                # Get the name of the tracer class.  Py3k has a different way to get it.
+                # Get the name of the tracer class.
                 try:
-                    trace_name = trace_fn.im_class.__name__
+                    trace_name = trace_fn.__self__.__class__.__name__
                 except AttributeError:
-                    try:
-                        trace_name = trace_fn.__self__.__class__.__name__
-                    except AttributeError:
-                        # A C-based function could also manifest as an f_trace value
-                        # which doesn't have im_class or __self__.
-                        trace_name = trace_fn.__class__.__name__
+                    # A C-based function could also manifest as an f_trace value
+                    # which doesn't have __self__.
+                    trace_name = trace_fn.__class__.__name__
 
             print(trace_name)
             """)
@@ -519,7 +534,7 @@ class ProcessTest(CoverageTest):
         assert py_out == "None\n"
 
         cov_out = self.run_command("coverage run showtrace.py")
-        if os.environ.get('COVERAGE_TEST_TRACER', 'c') == 'c':
+        if os.getenv('COVERAGE_TEST_TRACER', 'c') == 'c':
             # If the C trace function is being tested, then regular running should have
             # the C function, which registers itself as f_trace.
             assert cov_out == "CTracer\n"
@@ -556,30 +571,6 @@ class ProcessTest(CoverageTest):
         )
         assert msg in out
 
-    @pytest.mark.expensive
-    @pytest.mark.skipif(not env.C_TRACER, reason="fullcoverage only works with the C tracer.")
-    @pytest.mark.skipif(env.METACOV, reason="Can't test fullcoverage when measuring ourselves")
-    def test_fullcoverage(self) -> None:
-        # fullcoverage is a trick to get stdlib modules measured from
-        # the very beginning of the process. Here we import os and
-        # then check how many lines are measured.
-        self.make_file("getenv.py", """\
-            import os
-            print("FOOEY == %s" % os.getenv("FOOEY"))
-            """)
-
-        fullcov = os.path.join(os.path.dirname(coverage.__file__), "fullcoverage")
-        self.set_environ("FOOEY", "BOO")
-        self.set_environ("PYTHONPATH", fullcov)
-        out = self.run_command("python -X frozen_modules=off -m coverage run -L getenv.py")
-        assert out == "FOOEY == BOO\n"
-        data = coverage.CoverageData()
-        data.read()
-        # The actual number of executed lines in os.py when it's
-        # imported is 120 or so.  Just running os.getenv executes
-        # about 5.
-        assert line_counts(data)['os.py'] > 50
-
     # Pypy passes locally, but fails in CI? Perhaps the version of macOS is
     # significant?  https://foss.heptapod.net/pypy/pypy/-/issues/3074
     @pytest.mark.skipif(env.PYPY, reason="PyPy is unreliable with this test")
@@ -666,8 +657,7 @@ class EnvironmentTest(CoverageTest):
     def assert_tryexecfile_output(self, expected: str, actual: str) -> None:
         """Assert that the output we got is a successful run of try_execfile.py.
 
-        `expected` and `actual` must be the same, modulo a few slight known
-        platform differences.
+        `expected` and `actual` must be the same.
 
         """
         # First, is this even credible try_execfile.py output?
@@ -697,7 +687,7 @@ class EnvironmentTest(CoverageTest):
         self.assert_tryexecfile_output(expected, actual)
 
     def test_coverage_run_dashm_is_like_python_dashm(self) -> None:
-        # These -m commands assume the coverage tree is on the path.
+        self.add_test_modules_to_pythonpath()
         expected = self.run_command("python -m process_test.try_execfile")
         actual = self.run_command("coverage run -m process_test.try_execfile")
         self.assert_tryexecfile_output(expected, actual)
@@ -733,7 +723,7 @@ class EnvironmentTest(CoverageTest):
         When imported by -m, a module's __name__ is __main__, but we need the
         --source machinery to know and respect the original name.
         """
-        # These -m commands assume the coverage tree is on the path.
+        self.add_test_modules_to_pythonpath()
         expected = self.run_command("python -m process_test.try_execfile")
         actual = self.run_command(
             "coverage run --source process_test.try_execfile -m process_test.try_execfile"
@@ -753,7 +743,7 @@ class EnvironmentTest(CoverageTest):
             [run]
             disable_warnings = module-not-measured
             """)
-        # These -m commands assume the coverage tree is on the path.
+        self.add_test_modules_to_pythonpath()
         expected = self.run_command("python -m process_test.try_execfile")
         actual = self.run_command(
             "coverage run --source process_test -m process_test.try_execfile"
@@ -775,6 +765,7 @@ class EnvironmentTest(CoverageTest):
             import process_test.try_execfile
             """)
 
+        self.add_test_modules_to_pythonpath()
         expected = self.run_command("python myscript")
         actual = self.run_command("coverage run --source process_test myscript")
         self.assert_tryexecfile_output(expected, actual)
